@@ -44,6 +44,23 @@
 
 namespace torrent {
 
+struct chunk_list_last_modified {
+  chunk_list_last_modified() : m_time(cachedTime) {}
+
+  void operator () (ChunkListNode* node) {
+    if (node->time_modified() < m_time && node->time_modified() != rak::timer())
+      m_time = node->time_modified();
+  }
+
+  rak::timer m_time;
+};
+
+struct chunk_list_sort_index {
+  bool operator () (ChunkListNode* node1, ChunkListNode* node2) {
+    return node1->index() < node2->index();
+  }
+};
+
 inline bool
 ChunkList::is_queued(ChunkListNode* node) {
   return std::find(m_queue.begin(), m_queue.end(), node) != m_queue.end();
@@ -51,9 +68,7 @@ ChunkList::is_queued(ChunkListNode* node) {
 
 bool
 ChunkList::has_chunk(size_type index, int prot) const {
-  return
-    Base::at(index).is_valid() &&
-    Base::at(index).chunk()->has_permissions(prot);
+  return Base::at(index).is_valid() && Base::at(index).chunk()->has_permissions(prot);
 }
 
 void
@@ -63,7 +78,7 @@ ChunkList::resize(size_type s) {
 
   Base::resize(s);
 
-  int index = 0;
+  uint32_t index = 0;
 
   for (iterator itr = begin(), last = end(); itr != last; ++itr, ++index)
     itr->set_index(index);
@@ -71,13 +86,23 @@ ChunkList::resize(size_type s) {
 
 void
 ChunkList::clear() {
-  if (!m_queue.empty())
-    throw internal_error("ChunkList::clear() m_queue could not be clear.");
+  // Don't do any sync'ing as whomever decided to shut down really
+  // doesn't care, so just de-reference all chunks in queue.
+  for (Queue::iterator itr = m_queue.begin(), last = m_queue.end(); itr != last; ++itr) {
+    if ((*itr)->references() != 1 || (*itr)->writable() != 1)
+      throw internal_error("ChunkList::clear() called but a node in the queue is still referenced.");
+    
+    (*itr)->dec_rw();
 
-  if (std::find_if(begin(), end(), std::mem_fun_ref(&ChunkListNode::references)) != end())
-    throw internal_error("ChunkList::clear() called but a valid node was found.");
+    delete (*itr)->chunk();
+    (*itr)->set_chunk(NULL);
+  }
 
-  if (std::find_if(begin(), end(), std::mem_fun_ref(&ChunkListNode::writable)) != end())
+  m_queue.clear();
+
+  if (std::find_if(begin(), end(), std::mem_fun_ref(&ChunkListNode::chunk)) != end() ||
+      std::find_if(begin(), end(), std::mem_fun_ref(&ChunkListNode::references)) != end() ||
+      std::find_if(begin(), end(), std::mem_fun_ref(&ChunkListNode::writable)) != end())
     throw internal_error("ChunkList::clear() called but a valid node was found.");
 
   Base::clear();
@@ -106,9 +131,14 @@ ChunkList::get(size_type index, bool writable) {
 
   node->inc_references();
 
-  if (writable)
+  if (writable) {
     node->inc_writable();
 
+    // Make sure that periodic syncing uses async on any subsequent
+    // changes even if it was triggered before this get.
+    node->set_sync_triggered(false);
+  }
+
   return ChunkHandle(node, writable);
 }
 
@@ -134,11 +164,13 @@ ChunkList::release(ChunkHandle handle) {
 	throw internal_error("ChunkList::release(...) tried to queue an already queued chunk.");
 
       // Only add those that have a modification time set?
+      //
+      // Only chunks that are not already in the queue will execute
+      // this branch.
       m_queue.push_back(handle.node());
 
     } else {
-      handle->dec_references();
-      handle->dec_writable();
+      handle->dec_rw();
     }
 
   } else {
@@ -148,64 +180,68 @@ ChunkList::release(ChunkHandle handle) {
 
       delete handle->chunk();
       handle->set_chunk(NULL);
-      
     }
 
     handle->dec_references();
   }
 }
 
-struct chunk_list_last_modified {
-  chunk_list_last_modified() : m_time(cachedTime) {}
-
-  void operator () (ChunkListNode* node) {
-    if (node->time_modified() < m_time && node->time_modified() != rak::timer())
-      m_time = node->time_modified();
-  }
-
-  rak::timer m_time;
-};
-
-inline void
-ChunkList::sync_chunk(ChunkListNode* node) {
+inline bool
+ChunkList::sync_chunk(ChunkListNode* node, int flags, bool cleanup) {
   if (node->references() <= 0 || node->writable() <= 0)
     throw internal_error("ChunkList::sync_chunk(...) got a node with invalid reference count.");
 
-  // Check return value?
-  node->chunk()->sync(MemoryChunk::sync_async);
+  // Using sync for the time being.
+  if (!node->chunk()->sync(flags))
+    return false;
 
-  node->dec_writable();
-  node->dec_references();
+  node->set_sync_triggered(true);
+
+  if (!cleanup)
+    return true;
+
+  node->dec_rw();
  
   if (node->references() == 0) {
     delete node->chunk();
     node->set_chunk(NULL);
   }
+
+  return true;
 }
 
-inline bool
-ChunkList::less_chunk_index(ChunkListNode* node1, ChunkListNode* node2) {
-  return node1->index() < node2->index();
-}
+unsigned int
+ChunkList::sync_all(int flags) {
+  std::sort(m_queue.begin(), m_queue.end());
 
-void
-ChunkList::sync_all() {
-  Queue::iterator split = std::partition(m_queue.begin(), m_queue.end(),
-					 rak::not_equal(1, std::mem_fun(&ChunkListNode::writable)));
+  unsigned int failed = 0;
+  Queue::iterator split = m_queue.begin();
+
+  for (Queue::iterator itr = split, last = m_queue.end(); itr != last; ++itr)
+    if (!sync_chunk(*itr, flags, true)) {
+      // Makes sure the failed chunks don't get erased.
+      std::iter_swap(itr, split++);
+      failed++;
+    }
 
-  std::for_each(split, m_queue.end(), std::ptr_fun(&ChunkList::sync_chunk));
   m_queue.erase(split, m_queue.end());
+  return failed;
 }
 
-void
+unsigned int
 ChunkList::sync_periodic() {
   if (std::find_if(m_queue.begin(), m_queue.end(), rak::equal(0, std::mem_fun(&ChunkListNode::writable)))
       != m_queue.end())
     throw internal_error("ChunkList::sync_periodic() found a chunk with writable == 0.");
 
-  // Chunks that still have writers are moved infront of split.
-  Queue::iterator split = std::partition(m_queue.begin(), m_queue.end(),
-					 rak::not_equal(1, std::mem_fun(&ChunkListNode::writable)));
+  // Chunks that still have writers are moved infront of split. Those
+  // chunks were once released by the last writer, and later grabbed
+  // again. This should be relatively rare and should not cause any
+  // problems.
+  //
+  // Using std::stable_partition to keep the index order relatively
+  // intact for quicker sorting.
+  Queue::iterator split = std::stable_partition(m_queue.begin(), m_queue.end(), rak::not_equal(1, std::mem_fun(&ChunkListNode::writable)));
 
   // Do various partitioning before sorting by index, so that when we
   // call sync we always increase the index of the chunk.
@@ -217,12 +253,33 @@ ChunkList::sync_periodic() {
 
   if (std::distance(split, m_queue.end()) < (difference_type)m_maxQueueSize &&
       std::for_each(split, m_queue.end(), chunk_list_last_modified()).m_time + rak::timer::from_seconds(m_maxTimeQueued) < cachedTime)
-    return;
+    return 0;
 
-  std::sort(split, m_queue.end(), std::ptr_fun(&ChunkList::less_chunk_index));
+  // The queue contains pointer of objects in a vector, so we can sort
+  // by the pointer value.
+  std::sort(split, m_queue.end());
+
+  unsigned int failed = 0;
+
+  for (Queue::iterator itr = split, last = m_queue.end(); itr != last; ++itr) {
+    // Do first async on the chunk, then sync at the next periodic
+    // call.
+    if (!(*itr)->sync_triggered()) {
+      if (!sync_chunk(*itr, MemoryChunk::sync_async, false))
+	failed++;
+
+      std::iter_swap(itr, split++);
+
+    } else {
+      if (!sync_chunk(*itr, MemoryChunk::sync_sync, true)) {
+	failed++;
+	std::iter_swap(itr, split++);
+      }
+    }
+  }
 
-  std::for_each(split, m_queue.end(), std::ptr_fun(&ChunkList::sync_chunk));
   m_queue.erase(split, m_queue.end());
+  return failed;
 }
 
 }
