@@ -36,6 +36,9 @@
 
 #include "config.h"
 
+#include <rak/error_number.h>
+#include <rak/file_stat.h>
+
 #include "torrent/exceptions.h"
 
 #include "file_meta.h"
@@ -44,7 +47,6 @@
 namespace torrent {
 
 EntryListNode::EntryListNode() :
-  m_fileMeta(NULL),
   m_position(0),
   m_size(0),
   m_completed(0),
@@ -53,18 +55,41 @@ EntryListNode::EntryListNode() :
 }
 
 bool
-EntryListNode::resize_file() const {
-  if (m_fileMeta == NULL)
-    throw internal_error("EntryListNode::resize_file() called but m_fileMeta == NULL.");
+EntryListNode::is_created() const {
+  rak::file_stat fs;
 
-  if (!m_fileMeta->prepare(MemoryChunk::prot_read))
+  // If we can't even get permission to do fstat, we might as well
+  // consider the file as not created. This function is to be used by
+  // the client to check that the torrent files are present and ok,
+  // rather than as a way to find out if it is starting on a blank
+  // slate.
+  if (!fs.update(m_fileMeta.get_path()))
+//     return rak::error_number::current() == rak::error_number::e_access;
     return false;
 
-  if (m_size == m_fileMeta->get_file().size())
+  return fs.is_regular();
+}
+
+bool
+EntryListNode::is_correct_size() const {
+  rak::file_stat fs;
+
+  if (!fs.update(m_fileMeta.get_path()))
+    return false;
+
+  return fs.is_regular() && fs.size() == m_size;
+}
+
+bool
+EntryListNode::resize_file() {
+  if (!m_fileMeta.prepare(MemoryChunk::prot_read))
+    return false;
+
+  if (m_size == m_fileMeta.get_file().size())
     return true;
 
-  if (!m_fileMeta->prepare(MemoryChunk::prot_read | MemoryChunk::prot_write) ||
-      !m_fileMeta->get_file().set_size(m_size))
+  if (!m_fileMeta.prepare(MemoryChunk::prot_read | MemoryChunk::prot_write) ||
+      !m_fileMeta.get_file().set_size(m_size))
     return false;
   
   // Not here... make it a setting of sorts?
