@@ -39,9 +39,9 @@
 
 #include <inttypes.h>
 #include <rak/ranges.h>
+#include <rak/partial_queue.h>
 
-#include "utils/bitfield.h"
-#include "utils/bitfield_counter.h"
+#include "torrent/bitfield.h"
 
 namespace torrent {
 
@@ -53,6 +53,7 @@ namespace torrent {
 // When updating Content::bitfield, make sure you update this bitfield
 // and unmark any chunks in Delegator.
 
+class ChunkStatistics;
 class PeerChunks;
 
 class ChunkSelector {
@@ -64,15 +65,14 @@ public:
   bool                empty() const                 { return size() == 0; }
   uint32_t            size() const                  { return m_bitfield.size_bits(); }
 
-  const BitField*     bitfield()                    { return &m_bitfield; }
-  BitFieldCounter*    bitfield_counter()            { return &m_bitfieldCounter; }
+  const Bitfield*     bitfield()                    { return &m_bitfield; }
 
   PriorityRanges*     high_priority()               { return &m_highPriority; }
   PriorityRanges*     normal_priority()             { return &m_normalPriority; }
 
   // Initialize doesn't update the priority cache, so it is as if it
   // has empty priority ranges.
-  void                initialize(BitField* bf);
+  void                initialize(Bitfield* bf, ChunkStatistics* cs);
   void                cleanup();
 
   // Call this once you've modified the bitfield or priorities to
@@ -80,27 +80,44 @@ public:
   // find.
   void                update_priorities();
 
-  uint32_t            find(PeerChunks* peerChunks, bool highPriority);
+  uint32_t            find(PeerChunks* pc, bool highPriority);
 
+  bool                is_wanted(uint32_t index) const;
+
+  // Call this to set the index as being downloaded, finished etc,
+  // thus ignored. Propably should find a better name for this.
   void                using_index(uint32_t index);
   void                not_using_index(uint32_t index);
 
-  void                insert_peer_chunks(PeerChunks* peerChunks);
-  void                erase_peer_chunks(PeerChunks* peerChunks);
+  // The caller must ensure that the chunk index is valid and has not
+  // been set already.
+  //
+  // The user only needs to call this when it needs to know whetever
+  // it should become interested, or if it is in the process of
+  // downloading.
+  //
+  // Returns whetever we're interested in that piece.
+  bool                received_have_chunk(PeerChunks* pc, uint32_t index);
 
 private:
-  inline uint32_t     search(const BitField* bf, PriorityRanges* ranges, uint32_t first, uint32_t last);
-  inline uint32_t     search_range(const BitField* bf, uint32_t first, uint32_t last);
-  inline uint32_t     search_byte(uint8_t wanted);
+  bool                search_linear(const Bitfield* bf, rak::partial_queue* pq, PriorityRanges* ranges, uint32_t first, uint32_t last);
+  inline bool         search_linear_range(const Bitfield* bf, rak::partial_queue* pq, uint32_t first, uint32_t last);
+  inline bool         search_linear_byte(rak::partial_queue* pq, uint32_t index, Bitfield::value_type wanted);
+
+//   inline uint32_t     search_rarest(const Bitfield* bf, PriorityRanges* ranges, uint32_t first, uint32_t last);
+//   inline uint32_t     search_rarest_range(const Bitfield* bf, uint32_t first, uint32_t last);
+//   inline uint32_t     search_rarest_byte(uint8_t wanted);
 
   void                advance_position();
 
-  BitField            m_bitfield;
-  BitFieldCounter     m_bitfieldCounter;
+  Bitfield            m_bitfield;
+  ChunkStatistics*    m_statistics;
   
   PriorityRanges      m_highPriority;
   PriorityRanges      m_normalPriority;
 
+  rak::partial_queue  m_sharedQueue;
+
   uint32_t            m_position;
 };
 
