@@ -38,6 +38,7 @@
 
 #include <algorithm>
 
+#include "download/download_info.h"
 #include "protocol/peer_info.h"
 #include "protocol/peer_connection_base.h"
 #include "torrent/exceptions.h"
@@ -54,7 +55,7 @@ ConnectionList::clear() {
 
 bool
 ConnectionList::insert(DownloadMain* d, const PeerInfo& p, const SocketFd& fd) {
-  if (std::find_if(begin(), end(), rak::equal(p, std::mem_fun(&PeerConnectionBase::get_peer))) != end() ||
+  if (std::find_if(begin(), end(), rak::equal_ptr(&p, std::mem_fun(&PeerConnectionBase::peer_info))) != end() ||
       size() >= m_maxSize)
     return false;
 
@@ -66,6 +67,8 @@ ConnectionList::insert(DownloadMain* d, const PeerInfo& p, const SocketFd& fd) {
   c->initialize(d, p, fd);
 
   Base::push_back(c);
+
+  m_info->set_accepting_new_peers(size() < m_maxSize);
   m_slotConnected(c);
 
   return true;
@@ -79,6 +82,8 @@ ConnectionList::erase(iterator pos) {
   value_type v = *pos;
 
   pos = Base::erase(pos);
+
+  m_info->set_accepting_new_peers(size() < m_maxSize);
   m_slotDisconnected(v);
 
   // Delete after the erase to ensure the connection doesn't get added
@@ -99,6 +104,8 @@ ConnectionList::erase(PeerConnectionBase* p) {
   // emited otherwise some listeners might do stuff with the
   // assumption that the connection will remain in the list.
   Base::erase(itr);
+
+  m_info->set_accepting_new_peers(size() < m_maxSize);
   m_slotDisconnected(p);
 
   // Delete after the erase to ensure the connection doesn't get added
@@ -118,6 +125,8 @@ ConnectionList::erase_remaining(iterator pos) {
     m_slotDisconnected(v);
     delete v;
   }
+
+  m_info->set_accepting_new_peers(size() < m_maxSize);
 }
 
 void
@@ -125,32 +134,31 @@ ConnectionList::erase_seeders() {
   erase_remaining(std::partition(begin(), end(), std::not1(std::mem_fun(&PeerConnectionBase::is_seeder))));
 }
 
-struct _ConnectionListCompLess {
+struct connection_list_less {
   bool operator () (const PeerConnectionBase* p1, const PeerConnectionBase* p2) const {
-    return p1->get_peer().get_socket_address() < p2->get_peer().get_socket_address();
+    return *p1->peer_info()->socket_address() < *p2->peer_info()->socket_address();
   }
 
-  bool operator () (const SocketAddress& sa1, const PeerConnectionBase* p2) const {
-    return sa1 < p2->get_peer().get_socket_address();
+  bool operator () (const rak::socket_address& sa1, const PeerConnectionBase* p2) const {
+    return sa1 < *p2->peer_info()->socket_address();
   }
 
-  bool operator () (const PeerConnectionBase* p1, const SocketAddress& sa2) const {
-    return p1->get_peer().get_socket_address() < sa2;
+  bool operator () (const PeerConnectionBase* p1, const rak::socket_address& sa2) const {
+    return *p1->peer_info()->socket_address() < sa2;
   }
 };
 
 ConnectionList::iterator
-ConnectionList::find(const SocketAddress& sa) {
-  return std::find_if(begin(), end(),
-		      rak::equal(sa, rak::on(std::mem_fun(&PeerConnectionBase::get_peer),
-					     std::mem_fun_ref<const torrent::SocketAddress&>(&PeerInfo::get_socket_address))));
+ConnectionList::find(const rak::socket_address& sa) {
+  return std::find_if(begin(), end(), rak::equal_ptr(&sa, rak::on(std::mem_fun(&PeerConnectionBase::peer_info),
+								  std::mem_fun<const rak::socket_address*>(&PeerInfo::socket_address))));
 }
 
 void
 ConnectionList::set_difference(AddressList* l) {
-  std::sort(begin(), end(), _ConnectionListCompLess());
+  std::sort(begin(), end(), connection_list_less());
 
-  l->erase(std::set_difference(l->begin(), l->end(), begin(), end(), l->begin(), _ConnectionListCompLess()),
+  l->erase(std::set_difference(l->begin(), l->end(), begin(), end(), l->begin(), connection_list_less()),
 	   l->end());
 }
 
@@ -159,4 +167,10 @@ ConnectionList::send_finished_chunk(uint32_t index) {
   std::for_each(begin(), end(), std::bind2nd(std::mem_fun(&PeerConnectionBase::receive_finished_chunk), index));
 }
 
+void
+ConnectionList::set_max_size(size_type v) { 
+  m_maxSize = v;
+  m_info->set_accepting_new_peers(size() < m_maxSize);
+}
+
 }
