@@ -34,13 +34,15 @@
 //           Skomakerveien 33
 //           3185 Skoppum, NORWAY
 
+// Fucked up ugly piece of hack, this code.
+
 #include "config.h"
 
 #include <inttypes.h>
 
 #include "torrent/exceptions.h"
 #include "protocol/peer_chunks.h"
-#include "utils/bitfield.h"
+#include "torrent/bitfield.h"
 
 #include "delegator_chunk.h"
 #include "delegator_reservee.h"
@@ -63,8 +65,20 @@ struct DelegatorCheckAffinity {
   unsigned int        m_index;
 };
 
+struct DelegatorCheckSeeder {
+  DelegatorCheckSeeder(Delegator* delegator, DelegatorPiece** target) :
+    m_delegator(delegator), m_target(target) {}
+
+  bool operator () (DelegatorChunk* d) {
+    return d->by_seeder() && (*m_target = m_delegator->delegate_piece(d)) != NULL;
+  }
+
+  Delegator*          m_delegator;
+  DelegatorPiece**    m_target;
+};
+
 struct DelegatorCheckPriority {
-  DelegatorCheckPriority(Delegator* delegator, DelegatorPiece** target, Priority::Type p, const BitField* bf) :
+  DelegatorCheckPriority(Delegator* delegator, DelegatorPiece** target, Priority::Type p, const Bitfield* bf) :
     m_delegator(delegator), m_target(target), m_priority(p), m_bitfield(bf) {}
 
   bool operator () (DelegatorChunk* d) {
@@ -77,12 +91,12 @@ struct DelegatorCheckPriority {
   Delegator*          m_delegator;
   DelegatorPiece**    m_target;
   Priority::Type      m_priority;
-  const BitField*     m_bitfield;
+  const Bitfield*     m_bitfield;
 };
 
 // TODO: Should this ensure we don't download pieces that are priority off?
 struct DelegatorCheckAggressive {
-  DelegatorCheckAggressive(Delegator* delegator, DelegatorPiece** target, uint16_t* o, const BitField* bf) :
+  DelegatorCheckAggressive(Delegator* delegator, DelegatorPiece** target, uint16_t* o, const Bitfield* bf) :
     m_delegator(delegator), m_target(target), m_overlapp(o), m_bitfield(bf) {}
 
   bool operator () (DelegatorChunk* d) {
@@ -100,12 +114,17 @@ struct DelegatorCheckAggressive {
   Delegator*          m_delegator;
   DelegatorPiece**    m_target;
   uint16_t*           m_overlapp;
-  const BitField*     m_bitfield;
+  const Bitfield*     m_bitfield;
 };
 
 void Delegator::clear() {
   for (Chunks::iterator itr = m_chunks.begin(), last = m_chunks.end(); itr != last; ++itr) {
     m_slotChunkDisable((*itr)->get_index());
+
+    // Since there seems to be a bug somewhere:
+    for (DelegatorChunk::iterator j = (*itr)->begin(), l = (*itr)->end(); j != l; ++j)
+      j->clear();
+
     delete *itr;
   }
 
@@ -130,9 +149,12 @@ Delegator::delegate(PeerChunks* peerChunks, int affinity) {
       != m_chunks.end())
     return target->create();
 
+  if (peerChunks->is_seeder() && (target = delegate_seeder(peerChunks)) != NULL)
+    return target->create();
+
   // High priority pieces.
   if (std::find_if(m_chunks.begin(), m_chunks.end(),
-		   DelegatorCheckPriority(this, &target, Priority::HIGH, peerChunks->bitfield()->base()))
+		   DelegatorCheckPriority(this, &target, Priority::HIGH, peerChunks->bitfield()))
       != m_chunks.end())
     return target->create();
 
@@ -142,7 +164,7 @@ Delegator::delegate(PeerChunks* peerChunks, int affinity) {
 
   // Normal priority pieces.
   if (std::find_if(m_chunks.begin(), m_chunks.end(),
-		   DelegatorCheckPriority(this, &target, Priority::NORMAL, peerChunks->bitfield()->base()))
+		   DelegatorCheckPriority(this, &target, Priority::NORMAL, peerChunks->bitfield()))
       != m_chunks.end())
     return target->create();
 
@@ -159,11 +181,28 @@ Delegator::delegate(PeerChunks* peerChunks, int affinity) {
   uint16_t overlapped = 5;
 
   std::find_if(m_chunks.begin(), m_chunks.end(),
-	       DelegatorCheckAggressive(this, &target, &overlapped, peerChunks->bitfield()->base()));
+	       DelegatorCheckAggressive(this, &target, &overlapped, peerChunks->bitfield()));
 
   return target ? target->create() : NULL;
 }
   
+DelegatorPiece*
+Delegator::delegate_seeder(PeerChunks* peerChunks) {
+  DelegatorPiece* target = NULL;
+
+  if (std::find_if(m_chunks.begin(), m_chunks.end(), DelegatorCheckSeeder(this, &target))
+      != m_chunks.end())
+    return target;
+
+  if ((target = new_chunk(peerChunks, true)))
+    return target;
+  
+  if ((target = new_chunk(peerChunks, false)))
+    return target;
+
+  return NULL;
+}
+
 void
 Delegator::finished(DelegatorReservee& r) {
   if (!r.is_valid() || r.get_parent()->is_finished())
@@ -175,14 +214,14 @@ Delegator::finished(DelegatorReservee& r) {
     throw internal_error("Delegator::finished(...) got reservee with parent == NULL");
 
   // Temporary exception, remove when the code is rock solid. (Hah, like it ever will be;)
-  if (all_finished(p->get_piece().get_index()))
+  if (all_finished(p->get_piece().index()))
     throw internal_error("Delegator::finished(...) called on an index that is already finished");
 
   p->clear();
   p->set_finished(true);
 
-  if (all_finished(p->get_piece().get_index()))
-    m_slotChunkDone(p->get_piece().get_index());
+  if (all_finished(p->get_piece().index()))
+    m_slotChunkDone(p->get_piece().index());
 }
 
 void
@@ -219,7 +258,10 @@ Delegator::new_chunk(PeerChunks* pc, bool highPriority) {
 		   rak::equal(index, std::mem_fun(&DelegatorChunk::get_index))) != m_chunks.end())
     throw internal_error("Delegator::new_chunk(...) received an index that is already delegated.");
 
-  m_chunks.push_back(new DelegatorChunk(index, m_slotChunkSize(index), block_size, highPriority ? Priority::HIGH : Priority::NORMAL));
+  DelegatorChunk* chunk = new DelegatorChunk(index, m_slotChunkSize(index), block_size, highPriority ? Priority::HIGH : Priority::NORMAL);
+  chunk->set_by_seeder(pc->is_seeder());
+
+  m_chunks.push_back(chunk);
   m_slotChunkEnable(index);
 
   return (*m_chunks.rbegin())->begin();
@@ -228,7 +270,7 @@ Delegator::new_chunk(PeerChunks* pc, bool highPriority) {
 DelegatorPiece*
 Delegator::find_piece(const Piece& p) {
   Chunks::iterator c = std::find_if(m_chunks.begin(), m_chunks.end(),
-				    rak::equal((unsigned int)p.get_index(), std::mem_fun(&DelegatorChunk::get_index)));
+				    rak::equal(p.index(), std::mem_fun(&DelegatorChunk::get_index)));
   
   if (c == m_chunks.end())
     return NULL;
