@@ -46,19 +46,20 @@ namespace torrent {
 
 class DelegatorChunk {
 public:
-  typedef DelegatorPiece* iterator;
+  typedef DelegatorPiece*       iterator;
   typedef const DelegatorPiece* const_iterator;
 
-  DelegatorChunk(unsigned int index,
-		 uint32_t size,
-		 uint32_t piece_length,
-		 Priority::Type p);
+  DelegatorChunk(unsigned int index, uint32_t size, uint32_t piece_length, Priority::Type p);
   ~DelegatorChunk()                                   { delete [] m_pieces; }
 
   unsigned int        get_index() const               { return m_index; }
   unsigned int        get_piece_count()               { return m_count; }
   Priority::Type      priority()                      { return m_priority; }
 
+  // Hack.
+  bool                by_seeder() const               { return m_bySeeder; }
+  void                set_by_seeder(bool state)       { m_bySeeder = state; }
+
   DelegatorPiece*     begin()                         { return m_pieces; }
   DelegatorPiece*     end()                           { return m_pieces + m_count; }
 
@@ -68,11 +69,13 @@ private:
   DelegatorChunk(const DelegatorChunk&);
   void operator= (const DelegatorChunk&);
 
-  unsigned int    m_index;
-  unsigned int    m_count;
+  unsigned int        m_index;
+  unsigned int        m_count;
 
-  Priority::Type  m_priority;
-  DelegatorPiece* m_pieces;
+  Priority::Type      m_priority;
+  DelegatorPiece*     m_pieces;
+
+  bool                m_bySeeder;
 };
 
 }
