@@ -43,7 +43,7 @@
 
 namespace torrent {
 
-class Bencode;
+class Object;
 class Content;
 class DownloadWrapper;
 class TrackerManager;
@@ -58,26 +58,26 @@ public:
 
   DownloadConstructor() : m_download(NULL), m_encodingList(NULL) {}
 
-  void                initialize(const Bencode& b);
+  void                initialize(const Object& b);
 
   void                set_download(DownloadWrapper* d)         { m_download = d; }
   void                set_encoding_list(const EncodingList* e) { m_encodingList = e; }
 
 private:  
-  void                parse_tracker(const Bencode& b);
-  void                parse_info(const Bencode& b);
+  void                parse_tracker(const Object& b);
+  void                parse_info(const Object& b);
 
-  void                add_tracker_group(const Bencode& b);
-  void                add_tracker_single(const Bencode& b, int group);
+  void                add_tracker_group(const Object& b);
+  void                add_tracker_single(const Object& b, int group);
 
-  static bool         is_valid_path_element(const Bencode& b);
-  static bool         is_invalid_path_element(const Bencode& b) { return !is_valid_path_element(b); }
+  static bool         is_valid_path_element(const Object& b);
+  static bool         is_invalid_path_element(const Object& b) { return !is_valid_path_element(b); }
 
-  void                parse_single_file(const Bencode& b);
-  void                parse_multi_files(const Bencode& b);
-  void                add_file(const Bencode& b);
+  void                parse_single_file(const Object& b);
+  void                parse_multi_files(const Object& b);
+  void                add_file(const Object& b);
 
-  inline Path         create_path(const Bencode::List& plist, const std::string enc);
+  inline Path         create_path(const Object::list_type& plist, const std::string enc);
   inline Path         choose_path(std::list<Path>* pathList);
 
   DownloadWrapper*    m_download;
