@@ -40,33 +40,36 @@
 #include <limits>
 
 #include "data/chunk_list.h"
+#include "download/download_info.h"
 #include "protocol/handshake_manager.h"
 #include "protocol/peer_connection_base.h"
-#include "torrent/exceptions.h"
-#include "tracker/tracker_info.h"
 #include "tracker/tracker_manager.h"
+#include "torrent/exceptions.h"
 
 #include "choke_manager.h"
 #include "chunk_selector.h"
+#include "chunk_statistics.h"
 #include "download_main.h"
 
 namespace torrent {
 
 DownloadMain::DownloadMain() :
+  m_info(new DownloadInfo),
+
   m_trackerManager(new TrackerManager()),
-  m_chokeManager(new ChokeManager(&this->m_connectionList)),
   m_chunkList(new ChunkList),
   m_chunkSelector(new ChunkSelector),
+  m_chunkStatistics(new ChunkStatistics),
 
   m_started(false),
   m_isOpen(false),
   m_endgame(false),
 
   m_uploadThrottle(NULL),
-  m_downloadThrottle(NULL),
+  m_downloadThrottle(NULL) {
 
-  m_upRate(60),
-  m_downRate(60) {
+  m_connectionList = new ConnectionList(m_info);
+  m_chokeManager = new ChokeManager(m_connectionList);
 
   m_delegator.slot_chunk_enable(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::using_index));
   m_delegator.slot_chunk_disable(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::not_using_index));
@@ -74,10 +77,6 @@ DownloadMain::DownloadMain() :
   m_delegator.slot_chunk_done(rak::make_mem_fun(this, &DownloadMain::receive_chunk_done));
   m_delegator.slot_chunk_size(rak::make_mem_fun(&m_content, &Content::chunk_index_size));
 
-  m_trackerManager->tracker_info()->slot_stat_down() = rak::make_mem_fun(&m_downRate, &Rate::total);
-  m_trackerManager->tracker_info()->slot_stat_up()   = rak::make_mem_fun(&m_upRate, &Rate::total);
-  m_trackerManager->tracker_info()->slot_stat_left() = rak::make_mem_fun(this, &DownloadMain::get_bytes_left);
-
   m_taskTrackerRequest.set_slot(rak::mem_fn(this, &DownloadMain::receive_tracker_request));
 
   m_chunkList->slot_create_chunk(rak::make_mem_fun(&m_content, &Content::create_chunk));
@@ -89,8 +88,12 @@ DownloadMain::~DownloadMain() {
 
   delete m_trackerManager;
   delete m_chokeManager;
+  delete m_connectionList;
+
+  delete m_chunkStatistics;
   delete m_chunkList;
   delete m_chunkSelector;
+  delete m_info;
 }
 
 void
@@ -98,9 +101,10 @@ DownloadMain::open() {
   if (is_open())
     throw internal_error("Tried to open a download that is already open");
 
-  m_content.open();
+  m_content.entry_list()->open();
 
   m_chunkList->resize(m_content.chunk_total());
+  m_chunkStatistics->initialize(m_content.chunk_total());
 
   m_isOpen = true;
 }
@@ -117,10 +121,13 @@ DownloadMain::close() {
 
   m_trackerManager->close();
   m_delegator.clear();
-  m_content.close();
+
+  m_content.clear();
+  m_content.entry_list()->close();
 
   // Clear the chunklist last as it requires all referenced chunks to
   // be released.
+  m_chunkStatistics->clear();
   m_chunkList->clear();
   m_chunkSelector->cleanup();
 }
@@ -135,11 +142,20 @@ void DownloadMain::start() {
   m_started = true;
   m_lastConnectedSize = 0;
 
+  // Reset the uploaded baseline every time we restart the download so
+  // that broken trackers get the right uploaded ratio.
+  m_info->set_uploaded_baseline(m_info->up_rate()->total());
   m_trackerManager->send_start();
 
   receive_connect_peers();
 }  
 
+struct peer_connection_socket_address : public std::unary_function<const PeerConnectionBase*, const rak::socket_address&> {
+  const rak::socket_address& operator () (const PeerConnectionBase* p) const {
+    return *p->peer_info()->socket_address();
+  }
+};
+
 void
 DownloadMain::stop() {
   if (!m_started)
@@ -153,10 +169,8 @@ DownloadMain::stop() {
   // perform alot of requests to the tracker when restarting the
   // torrent. Consider saving these in the torrent file when dumping
   // it.
-  std::list<SocketAddress> addressList;
-
-  std::transform(connection_list()->begin(), connection_list()->end(), std::back_inserter(addressList),
-		 rak::on(std::mem_fun(&PeerConnectionBase::get_peer), std::mem_fun_ref<const torrent::SocketAddress&>(&PeerInfo::get_socket_address)));
+  std::list<rak::socket_address> addressList;
+  std::transform(connection_list()->begin(), connection_list()->end(), std::back_inserter(addressList), peer_connection_socket_address());
 
   addressList.sort();
   available_list()->insert(&addressList);
@@ -164,10 +178,7 @@ DownloadMain::stop() {
   while (!connection_list()->empty())
     connection_list()->erase(connection_list()->front());
 
-  m_trackerManager->send_stop();
   priority_queue_erase(&taskScheduler, &m_taskTrackerRequest);
-
-  m_chunkList->sync_all();
 }
 
 uint64_t
@@ -209,13 +220,11 @@ DownloadMain::receive_connect_peers() {
 
   while (!available_list()->empty() &&
 	 connection_list()->size() < connection_list()->get_min_size() &&
-	 connection_list()->size() + m_slotCountHandshakes(tracker_manager()->tracker_info()->get_hash()) < connection_list()->get_max_size()) {
-    SocketAddress sa = available_list()->pop_random();
+	 connection_list()->size() + m_slotCountHandshakes(m_info) < connection_list()->get_max_size()) {
+    rak::socket_address sa = available_list()->pop_random();
 
     if (connection_list()->find(sa) == connection_list()->end())
-      m_slotStartHandshake(sa,
-			   tracker_manager()->tracker_info()->get_hash(),
-			   tracker_manager()->tracker_info()->get_local_id());
+      m_slotStartHandshake(sa, m_info);
   }
 }
 
