@@ -48,16 +48,20 @@
 #include "data/chunk_handle.h"
 #include "protocol/peer_info.h"
 #include "globals.h"
-#include "torrent/rate.h"
+
+#include "download/download_info.h"
 
 namespace torrent {
 
 class ChunkList;
-class ChokeManager;
 class ChunkSelector;
+class ChunkStatistics;
+
+class ChokeManager;
 class DownloadWrapper;
 class HandshakeManager;
 class TrackerManager;
+class DownloadInfo;
 class ThrottleList;
 
 class DownloadMain {
@@ -78,14 +82,18 @@ public:
   TrackerManager*     tracker_manager()                          { return m_trackerManager; }
   TrackerManager*     tracker_manager() const                    { return m_trackerManager; }
 
-  // Only retrive writable chunks when the download is active.
+  DownloadInfo*       info()                                     { return m_info; }
+
+  // Only retrieve writable chunks when the download is active.
   ChunkList*          chunk_list()                               { return m_chunkList; }
   ChunkSelector*      chunk_selector()                           { return m_chunkSelector; }
+  ChunkStatistics*    chunk_statistics()                         { return m_chunkStatistics; }
+  
   Content*            content()                                  { return &m_content; }
   Delegator*          delegator()                                { return &m_delegator; }
 
   AvailableList*      available_list()                           { return &m_availableList; }
-  ConnectionList*     connection_list()                          { return &m_connectionList; }
+  ConnectionList*     connection_list()                          { return m_connectionList; }
 
   ThrottleList*       upload_throttle()                          { return m_uploadThrottle; }
   void                set_upload_throttle(ThrottleList* t)       { m_uploadThrottle = t; }
@@ -96,23 +104,13 @@ public:
   bool                get_endgame() const                        { return m_endgame; }
   uint64_t            get_bytes_left() const;
 
-  Rate*               up_rate()                                  { return &m_upRate; }
-  Rate*               down_rate()                                { return &m_downRate; }
-
   // Carefull with these.
   void                setup_delegator();
   void                setup_tracker();
 
-  typedef sigc::signal1<void, const std::string&>                SignalString;
-  typedef sigc::signal1<void, uint32_t>                          SignalChunk;
-
-  typedef rak::mem_fun3<HandshakeManager, void, const SocketAddress&,
-		       const std::string&, const std::string&>         SlotStartHandshake;
-  typedef rak::mem_fun1<HandshakeManager, uint32_t, const std::string&> SlotCountHandshakes;
-  typedef rak::mem_fun1<DownloadWrapper, void, ChunkHandle>             SlotHashCheckAdd;
-
-  SignalString&       signal_network_log()                       { return m_signalNetworkLog; }
-  SignalString&       signal_storage_error()                     { return m_signalStorageError; }
+  typedef rak::mem_fun2<HandshakeManager, void, const rak::socket_address&, DownloadInfo*> SlotStartHandshake;
+  typedef rak::const_mem_fun1<HandshakeManager, uint32_t, DownloadInfo*>                   SlotCountHandshakes;
+  typedef rak::mem_fun1<DownloadWrapper, void, ChunkHandle>                                SlotHashCheckAdd;
 
   void                slot_start_handshake(SlotStartHandshake s)   { m_slotStartHandshake = s; }
   void                slot_count_handshakes(SlotCountHandshakes s) { m_slotCountHandshakes = s; }
@@ -134,16 +132,20 @@ private:
   void                setup_start();
   void                setup_stop();
 
+  DownloadInfo*       m_info;
+
   TrackerManager*     m_trackerManager;
   ChokeManager*       m_chokeManager;
 
   ChunkList*          m_chunkList;
   ChunkSelector*      m_chunkSelector;
+  ChunkStatistics*    m_chunkStatistics;
+
   Content             m_content;
   Delegator           m_delegator;
 
   AvailableList       m_availableList;
-  ConnectionList      m_connectionList;
+  ConnectionList*     m_connectionList;
 
   bool                m_started;
   bool                m_isOpen;
@@ -153,12 +155,6 @@ private:
   ThrottleList*       m_uploadThrottle;
   ThrottleList*       m_downloadThrottle;
 
-  Rate                m_upRate;
-  Rate                m_downRate;
-
-  SignalString        m_signalNetworkLog;
-  SignalString        m_signalStorageError;
-
   SlotStartHandshake  m_slotStartHandshake;
   SlotCountHandshakes m_slotCountHandshakes;
   SlotHashCheckAdd    m_slotHashCheckAdd;
