@@ -42,6 +42,7 @@
 namespace torrent {
 
 class DownloadWrapper;
+class DownloadInfo;
 
 class DownloadManager : private std::list<DownloadWrapper*>{
 public:
@@ -75,6 +76,8 @@ public:
   void                clear();
 
   iterator            find(const std::string& hash);
+  iterator            find(DownloadInfo* info);
+  DownloadInfo*       find_info(const std::string& hash);
 };
 
 }
