@@ -36,6 +36,7 @@
 
 #include "config.h"
 
+#include <iterator>
 #include <stdlib.h>
 #include <rak/file_stat.h>
 #include <sigc++/bind.h>
@@ -49,6 +50,7 @@
 #include "protocol/handshake_manager.h"
 #include "protocol/peer_connection_base.h"
 #include "torrent/exceptions.h"
+#include "torrent/object.h"
 #include "tracker/tracker_manager.h"
 
 #include "chunk_selector.h"
@@ -58,10 +60,15 @@
 namespace torrent {
 
 DownloadWrapper::DownloadWrapper() :
+  m_bencode(NULL),
   m_hash(NULL),
   m_connectionType(0) {
 
   m_delayDownloadDone.set_slot(rak::mem_fn(&m_signalDownloadDone, &Signal::operator()));
+
+  m_main.tracker_manager()->set_info(info());
+  m_main.tracker_manager()->slot_success(rak::make_mem_fun(this, &DownloadWrapper::receive_tracker_success));
+  m_main.tracker_manager()->slot_failed(rak::make_mem_fun(this, &DownloadWrapper::receive_tracker_failed));
 }
 
 DownloadWrapper::~DownloadWrapper() {
@@ -72,17 +79,17 @@ DownloadWrapper::~DownloadWrapper() {
     close();
 
   delete m_hash;
+  delete m_bencode;
 }
 
 void
 DownloadWrapper::initialize(const std::string& hash, const std::string& id) {
   m_main.slot_hash_check_add(rak::make_mem_fun(this, &DownloadWrapper::check_chunk_hash));
 
-  m_main.tracker_manager()->tracker_info()->set_hash(hash);
-  m_main.tracker_manager()->tracker_info()->set_local_id(id);
-  m_main.tracker_manager()->tracker_info()->set_key(random());
-  m_main.tracker_manager()->slot_success(rak::make_mem_fun(this, &DownloadWrapper::receive_tracker_success));
-  m_main.tracker_manager()->slot_failed(rak::make_mem_fun(this, &DownloadWrapper::receive_tracker_failed));
+  info()->set_hash(hash);
+  info()->set_local_id(id);
+
+  info()->slot_stat_left() = rak::make_mem_fun(&m_main, &DownloadMain::get_bytes_left);
 
   m_main.chunk_list()->set_max_queue_size((32 << 20) / m_main.content()->chunk_size());
 
@@ -103,39 +110,33 @@ DownloadWrapper::hash_resume_load() {
   if (!m_main.is_open() || m_main.is_active() || m_hash->is_checked())
     throw client_error("DownloadWrapper::resume_load() called with wrong state");
 
-  if (!m_bencode.has_key("libtorrent resume"))
+  if (!m_bencode->has_key_map("libtorrent resume"))
     return;
 
   try {
-    Bencode& resume  = m_bencode.get_key("libtorrent resume");
+    Object& resume  = m_bencode->get_key("libtorrent resume");
 
     // Load peer addresses.
-    if (resume.has_key("peers") && resume.get_key("peers").is_string()) {
-      const std::string& peers = resume.get_key("peers").as_string();
+    if (resume.has_key_string("peers"))
+      insert_available_list(resume.get_key("peers").as_string());
 
-      std::list<SocketAddress> l(reinterpret_cast<const SocketAddressCompact*>(peers.c_str()),
-				 reinterpret_cast<const SocketAddressCompact*>(peers.c_str() + peers.size() - peers.size() % sizeof(SocketAddressCompact)));
+    Object& files = resume.get_key("files");
 
-      l.sort();
-      m_main.available_list()->insert(&l);
+    if (resume.has_key_string("bitfield") &&
+	resume.get_key("bitfield").as_string().size() == m_main.content()->bitfield()->size_bytes() &&
+	files.as_list().size() == m_main.content()->entry_list()->files_size()) {
+
+      // Clear the hash checking ranges, and add the files ranges we
+      // must check.
+      m_hash->ranges().clear();
+      m_main.content()->bitfield()->from_c_str(resume.get_key("bitfield").as_string().c_str());
+
+    } else {
+      // No resume bitfield available, check the whole range.
+      m_hash->ranges().insert(0, m_main.content()->chunk_total());
     }
 
-    if (resume.has_key("total_uploaded") && resume.get_key("total_uploaded").is_value())
-      m_main.up_rate()->set_total(resume.get_key("total_uploaded").as_value());
-
-    Bencode& files = resume.get_key("files");
-
-    if (resume.get_key("bitfield").as_string().size() != m_main.content()->bitfield().size_bytes() ||
-	files.as_list().size() != m_main.content()->entry_list()->files_size())
-      // FIXME: Better control logic.
-      throw bencode_error("");
-
-    // Clear the hash checking ranges, and add the files ranges we must check.
-    m_hash->ranges().clear();
-
-    std::memcpy(m_main.content()->bitfield().begin(), resume.get_key("bitfield").as_string().c_str(), m_main.content()->bitfield().size_bytes());
-
-    Bencode::List::iterator bItr = files.as_list().begin();
+    Object::list_type::iterator bItr = files.as_list().begin();
     EntryList::iterator sItr = m_main.content()->entry_list()->begin();
 
     // Check the validity of each file, add to the m_hash's ranges if invalid.
@@ -145,19 +146,17 @@ DownloadWrapper::hash_resume_load() {
       // Check that the size and modified stamp matches. If not, then
       // add to the hashes to check.
 
-      if (!fs.update(sItr->file_meta()->get_path()) ||
-	  sItr->size() != fs.size() ||
-	  !bItr->has_key("mtime") ||
-	  !(*bItr).get_key("mtime").is_value() ||
-	  (*bItr).get_key("mtime").as_value() != fs.modified_time())
-	m_hash->ranges().insert(sItr->range().first, sItr->range().second);
+      if (!fs.update((*sItr)->file_meta()->get_path()) ||
+	  (*sItr)->size() != fs.size() ||
+	  !bItr->has_key_value("mtime") ||
+	  bItr->get_key("mtime").as_value() != fs.modified_time())
+	m_hash->ranges().insert((*sItr)->range().first, (*sItr)->range().second);
 
       // Update the priority from the fast resume data.
-      if (bItr->has_key("priority") &&
-	  (*bItr).get_key("priority").is_value() &&
-	  (*bItr).get_key("priority").as_value() >= 0 &&
-	  (*bItr).get_key("priority").as_value() < 3)
-	sItr->set_priority((*bItr).get_key("priority").as_value());
+      if (bItr->has_key_value("priority") &&
+	  bItr->get_key("priority").as_value() >= 0 &&
+	  bItr->get_key("priority").as_value() < 3)
+	(*sItr)->set_priority((*bItr).get_key("priority").as_value());
 
       ++sItr;
       ++bItr;
@@ -168,8 +167,12 @@ DownloadWrapper::hash_resume_load() {
   }
 
   // Clear bits in invalid regions which will be checked by m_hash.
+  //
+  // If this is ever optimized in such a way as not to update the
+  // bitfield's set size, make sure that Content::update_done() calls
+  // Bitfield::update().
   for (HashTorrent::Ranges::iterator itr = m_hash->ranges().begin(); itr != m_hash->ranges().end(); ++itr)
-    m_main.content()->bitfield().set(itr->first, itr->second, false);
+    m_main.content()->bitfield()->unset_range(itr->first, itr->second);
 
   m_main.content()->update_done();
 }
@@ -186,37 +189,35 @@ DownloadWrapper::hash_resume_save() {
     // valid, just that the client didn't finish the check this time.
     return;
 
+  // If we can't sync, don't save the resume data.
+  if (m_main.chunk_list()->sync_all(MemoryChunk::sync_sync) != 0 && m_bencode->has_key("libtorrent resume")) {
+    m_bencode->get_key("libtorrent resume").erase_key("bitfield");
+    return;
+  }
+
   // Clear the resume data since if the syncing fails we propably don't
   // want the old resume data.
-  Bencode& resume = m_bencode.insert_key("libtorrent resume", Bencode(Bencode::TYPE_MAP));
+  Object& resume = m_bencode->insert_key("libtorrent resume", Object(Object::TYPE_MAP));
 
-  // We're guaranteed that file modification time is correctly updated
-  // after this. Though won't help if the files have been delete while
-  // we had them open.
-  //m_main.content()->entry_list()->sync_all();
+  resume.insert_key("bitfield", std::string((char*)m_main.content()->bitfield()->begin(), m_main.content()->bitfield()->size_bytes()));
 
-  // We sync all chunks in DownloadMain::stop(), so we are guaranteed
-  // that it has been called when we arrive here.
-
-  resume.insert_key("bitfield", std::string((char*)m_main.content()->bitfield().begin(), m_main.content()->bitfield().size_bytes()));
-
-  Bencode::List& l = resume.insert_key("files", Bencode(Bencode::TYPE_LIST)).as_list();
+  Object::list_type& l = resume.insert_key("files", Object(Object::TYPE_LIST)).as_list();
 
   EntryList::iterator sItr = m_main.content()->entry_list()->begin();
   
   // Check the validity of each file, add to the m_hash's ranges if invalid.
   while (sItr != m_main.content()->entry_list()->end()) {
-    Bencode& b = *l.insert(l.end(), Bencode(Bencode::TYPE_MAP));
+    Object& b = *l.insert(l.end(), Object(Object::TYPE_MAP));
 
     rak::file_stat fs;
 
-    if (!fs.update(sItr->file_meta()->get_path())) {
+    if (!fs.update((*sItr)->file_meta()->get_path())) {
       l.clear();
       break;
     }
 
     b.insert_key("mtime", fs.modified_time());
-    b.insert_key("priority", (int)sItr->priority());
+    b.insert_key("priority", (int)(*sItr)->priority());
 
     ++sItr;
   }
@@ -224,17 +225,10 @@ DownloadWrapper::hash_resume_save() {
   // Save the available peer list. Since this function is called when
   // the download is stopped, we know that all the previously
   // connected peers have been copied to the available list.
-  std::string peers;
-  peers.reserve(m_main.available_list()->size() * sizeof(SocketAddressCompact));
-  
-  for (AvailableList::const_iterator
-	 itr = m_main.available_list()->begin(),
-	 last = m_main.available_list()->end();
-       itr != last; ++itr)
-    peers.append(itr->get_address_compact().c_str(), sizeof(SocketAddressCompact));
-
-  resume.insert_key("peers", peers);
-  resume.insert_key("total_uploaded", m_main.up_rate()->total());
+  //
+  // Consider whetever we need to add currently connected too, as we
+  // might save the session while still active.
+  extract_available_list(&resume.insert_key("peers", Object()));
 }
 
 void
@@ -254,7 +248,12 @@ DownloadWrapper::close() {
 
   // Clear after m_hash to ensure that the empty hash done signal does
   // not get passed to HashTorrent.
-  m_hash->get_queue()->remove(get_hash());
+  m_hash->get_queue()->remove(info()->hash());
+
+  // This could/should be async as we do not care that much if it
+  // succeeds or not, any chunks not included in that last
+  // hash_resume_save get ignored anyway.
+  m_main.chunk_list()->sync_all(MemoryChunk::sync_async);
 
   m_main.close();
 
@@ -279,15 +278,15 @@ DownloadWrapper::start() {
   m_main.start();
 }
 
+// Remember to update receive_tick() when changing stop().
 void
 DownloadWrapper::stop() {
-  if (!m_main.is_active())
-    return;
-
   m_main.stop();
 
   m_connectionChunkPassed.disconnect();
   m_connectionChunkFailed.disconnect();
+
+  m_main.tracker_manager()->send_stop();
 }
 
 bool
@@ -295,41 +294,32 @@ DownloadWrapper::is_stopped() const {
   return !m_main.tracker_manager()->is_active();
 }
 
-const std::string&
-DownloadWrapper::get_hash() const {
-  return m_main.tracker_manager()->tracker_info()->get_hash();
-}
+void
+DownloadWrapper::insert_available_list(const std::string& src) {
+  std::list<rak::socket_address> l;
 
-const std::string&
-DownloadWrapper::get_local_id() const {
-  return m_main.tracker_manager()->tracker_info()->get_local_id();
-}
+  std::copy(reinterpret_cast<const SocketAddressCompact*>(src.c_str()),
+	    reinterpret_cast<const SocketAddressCompact*>(src.c_str() + src.size() - src.size() % sizeof(SocketAddressCompact)),
+	    std::back_inserter(l));
+  l.sort();
 
-SocketAddress&
-DownloadWrapper::bind_address() {
-  return m_main.tracker_manager()->tracker_info()->bind_address();
-}
-
-SocketAddress&
-DownloadWrapper::local_address() {
-  return m_main.tracker_manager()->tracker_info()->local_address();
+  m_main.available_list()->insert(&l);
 }
 
 void
-DownloadWrapper::set_file_manager(FileManager* f) {
-  m_main.content()->entry_list()->slot_insert_filemeta(rak::make_mem_fun(f, &FileManager::insert));
-  m_main.content()->entry_list()->slot_erase_filemeta(rak::make_mem_fun(f, &FileManager::erase));
-}
+DownloadWrapper::extract_available_list(Object* dest) {
+  *dest = std::string();
+  
+  std::string& peers = dest->as_string();
+  peers.reserve(m_main.available_list()->size() * sizeof(SocketAddressCompact));
+  
+  for (AvailableList::const_iterator itr = m_main.available_list()->begin(), last = m_main.available_list()->end(); itr != last; ++itr) {
+    if (itr->family() == rak::socket_address::af_inet) {
+      SocketAddressCompact sac(itr->sa_inet()->address_n(), itr->sa_inet()->port_n());
 
-void
-DownloadWrapper::set_handshake_manager(HandshakeManager* h) {
-  m_main.slot_count_handshakes(rak::make_mem_fun(h, &HandshakeManager::size_hash));
-  m_main.slot_start_handshake(rak::make_mem_fun(h, &HandshakeManager::add_outgoing));
-}
-
-void
-DownloadWrapper::set_hash_queue(HashQueue* h) {
-  m_hash->set_queue(h);
+      peers.append(sac.c_str(), sizeof(SocketAddressCompact));
+    }
+  }  
 }
 
 void
@@ -351,7 +341,7 @@ DownloadWrapper::receive_initial_hash() {
 
     // Clear after m_hash to ensure that the empty hash done signal does
     // not get passed to HashTorrent.
-    m_hash->get_queue()->remove(get_hash());
+    m_hash->get_queue()->remove(info()->hash());
     m_main.content()->clear();
 
   } else if (!m_main.content()->entry_list()->resize_all()) {
@@ -361,12 +351,12 @@ DownloadWrapper::receive_initial_hash() {
     // Do we clear the hash?
   }
 
-  if (m_hash->get_queue()->has(get_hash()))
+  if (m_hash->get_queue()->has(info()->hash()))
     throw internal_error("DownloadWrapper::receive_initial_hash() found a chunk in the HashQueue.");
 
   // Initialize the ChunkSelector here so that no chunks will be
   // marked by HashTorrent that are not accounted for.
-  m_main.chunk_selector()->initialize(&m_main.content()->bitfield());
+  m_main.chunk_selector()->initialize(m_main.content()->bitfield(), m_main.chunk_statistics());
   receive_update_priorities();
 
   m_signalInitialHash.emit();
@@ -375,7 +365,7 @@ DownloadWrapper::receive_initial_hash() {
 void
 DownloadWrapper::check_chunk_hash(ChunkHandle handle) {
   // Using HashTorrent's queue temporarily.
-  m_hash->get_queue()->push_back(handle, rak::make_mem_fun(this, &DownloadWrapper::receive_hash_done), get_hash());
+  m_hash->get_queue()->push_back(handle, rak::make_mem_fun(this, &DownloadWrapper::receive_hash_done), info()->hash());
 }
 
 void
@@ -386,26 +376,31 @@ DownloadWrapper::receive_hash_done(ChunkHandle handle, std::string h) {
   if (!is_open())
     throw internal_error("DownloadWrapper::receive_hash_done(...) called but the download is not open.");
 
+  uint32_t index = handle->index();
+  m_main.chunk_list()->release(handle);
+
   if (m_hash->is_checking()) {
-    m_main.content()->receive_chunk_hash(handle->index(), h);
+    m_main.content()->receive_chunk_hash(index, h);
     m_hash->receive_chunkdone();
 
   } else if (m_hash->is_checked()) {
 
-    if (m_main.chunk_selector()->bitfield()->get(handle->index()))
+    if (m_main.chunk_selector()->bitfield()->get(index))
       throw internal_error("DownloadWrapper::receive_hash_done(...) received a chunk that isn't set in ChunkSelector.");
 
-    if (m_main.content()->receive_chunk_hash(handle->index(), h)) {
-      signal_chunk_passed().emit(handle->index());
+    if (m_main.content()->receive_chunk_hash(index, h)) {
+
+      // Should this be done here, or after send_finished_chunk?
+      signal_chunk_passed().emit(index);
       m_main.update_endgame();
 
       if (m_main.content()->is_done())
 	finished_download();
     
-      m_main.connection_list()->send_finished_chunk(handle->index());
+      m_main.connection_list()->send_finished_chunk(index);
 
     } else {
-      signal_chunk_failed().emit(handle->index());
+      signal_chunk_failed().emit(index);
     }
 
   } else {
@@ -415,13 +410,11 @@ DownloadWrapper::receive_hash_done(ChunkHandle handle, std::string h) {
 
     //throw internal_error("DownloadWrapper::receive_hash_done(...) called but we're not in the right state.");
   }
-
-  m_main.chunk_list()->release(handle);
 }  
 
 void
 DownloadWrapper::receive_storage_error(const std::string& str) {
-  m_main.signal_storage_error().emit(str);
+  info()->signal_storage_error().emit(str);
 }
 
 void
@@ -453,7 +446,20 @@ DownloadWrapper::receive_peer_disconnected(PeerConnectionBase* peer) {
 
 void
 DownloadWrapper::receive_tick() {
-  m_main.chunk_list()->sync_periodic();
+  if (!m_main.is_open())
+    return;
+
+  unsigned int syncFailed = m_main.chunk_list()->sync_periodic();
+
+  if (m_main.is_active() && syncFailed != 0) {
+    // Need to move this stuff into a seperate function.
+    m_main.stop();
+
+    m_connectionChunkPassed.disconnect();
+    m_connectionChunkFailed.disconnect();
+
+    info()->signal_storage_error().emit("Could not sync data to disk, possibly full.");
+  }
 }
 
 void
@@ -465,11 +471,11 @@ DownloadWrapper::receive_update_priorities() {
   m_main.chunk_selector()->normal_priority()->clear();
 
   for (EntryList::iterator itr = m_main.content()->entry_list()->begin(); itr != m_main.content()->entry_list()->end(); ++itr) {
-    if (itr->priority() == 1)
-      m_main.chunk_selector()->normal_priority()->insert(itr->range().first, itr->range().second);
+    if ((*itr)->priority() == 1)
+      m_main.chunk_selector()->normal_priority()->insert((*itr)->range().first, (*itr)->range().second);
 
-    else if (itr->priority() == 2)
-      m_main.chunk_selector()->high_priority()->insert(itr->range().first, itr->range().second);
+    else if ((*itr)->priority() == 2)
+      m_main.chunk_selector()->high_priority()->insert((*itr)->range().first, (*itr)->range().second);
   }
 
   m_main.chunk_selector()->update_priorities();
@@ -491,7 +497,7 @@ DownloadWrapper::finished_download() {
     priority_queue_insert(&taskScheduler, &m_delayDownloadDone, cachedTime);
 
   m_main.connection_list()->erase_seeders();
-  m_main.down_rate()->reset_rate();
+  info()->down_rate()->reset_rate();
 }
 
 }
