@@ -37,14 +37,13 @@
 #ifndef LIBTORRENT_DOWNLOAD_WRAPPER_H
 #define LIBTORRENT_DOWNLOAD_WRAPPER_H
 
+#include <rak/socket_address.h>
 #include <sigc++/connection.h>
 #include <sigc++/signal.h>
 
 #include "data/chunk_handle.h"
-#include "net/socket_address.h"
-#include "torrent/bencode.h"
 #include "torrent/peer.h"
-#include "tracker/tracker_info.h"
+#include "download/download_info.h"
 
 #include "download_main.h"
 
@@ -56,10 +55,12 @@ class FileManager;
 class HashQueue;
 class HashTorrent;
 class HandshakeManager;
+class DownloadInfo;
+class Object;
 
 class DownloadWrapper {
 public:
-  typedef std::list<SocketAddress>                AddressList;
+  typedef std::list<rak::socket_address>          AddressList;
 
   typedef sigc::signal0<void>                     Signal;
   typedef sigc::signal1<void, uint32_t>           SignalChunk;
@@ -82,30 +83,23 @@ public:
   void                start();
   void                stop();
 
-  bool                is_open() const                { return m_main.is_open(); }
+  bool                is_open() const                         { return m_main.is_open(); }
   bool                is_stopped() const;
 
-  DownloadMain*       main()                         { return &m_main; }
-  const DownloadMain* main() const                   { return &m_main; }
-  HashTorrent*        hash_checker()                 { return m_hash; }
+  DownloadMain*       main()                                  { return &m_main; }
+  const DownloadMain* main() const                            { return &m_main; }
+  HashTorrent*        hash_checker()                          { return m_hash; }
 
-  Bencode&            bencode()                      { return m_bencode; }
+  Object*             bencode()                               { return m_bencode; }
+  void                set_bencode(Object* o)                  { m_bencode = o; }
 
-  const std::string&  get_hash() const;
-  const std::string&  get_local_id() const;
+  DownloadInfo*       info()                                  { return m_main.info(); }
 
-  SocketAddress&      bind_address();
-  SocketAddress&      local_address();
+  int                 connection_type() const                 { return m_connectionType; }
+  void                set_connection_type(int t)              { m_connectionType = t; }
 
-  const std::string&  get_name() const               { return m_name; }
-  void                set_name(const std::string& s) { m_name = s; }
-
-  void                set_file_manager(FileManager* f);
-  void                set_handshake_manager(HandshakeManager* h);
-  void                set_hash_queue(HashQueue* h);
-
-  int                 get_connection_type() const    { return m_connectionType; }
-  void                set_connection_type(int t)     { m_connectionType = t; }
+  void                insert_available_list(const std::string& src);
+  void                extract_available_list(Object* dest);
 
   void                receive_keepalive();
   void                receive_initial_hash();
@@ -144,15 +138,15 @@ private:
   void                finished_download();
 
   DownloadMain        m_main;
-  Bencode             m_bencode;
+  Object*             m_bencode;
   HashTorrent*        m_hash;
 
-  std::string         m_name;
   int                 m_connectionType;
 
+  rak::priority_item  m_delayDownloadDone;
+
   Signal              m_signalInitialHash;
   Signal              m_signalDownloadDone;
-  rak::priority_item  m_delayDownloadDone;
 
   Signal              m_signalTrackerSuccess;
   SignalString        m_signalTrackerFailed;
