@@ -46,9 +46,10 @@
 #include "data/hash_torrent.h"
 #include "protocol/handshake_manager.h"
 #include "data/hash_queue.h"
-#include "net/listen.h"
-#include "net/manager.h"
 #include "net/throttle_manager.h"
+#include "net/listen.h"
+
+#include "torrent/connection_manager.h"
 
 #include "manager.h"
 #include "resource_manager.h"
@@ -62,9 +63,12 @@ Manager::Manager() :
   m_fileManager(new FileManager),
   m_handshakeManager(new HandshakeManager),
   m_hashQueue(new HashQueue),
-  m_listen(new Listen),
   m_resourceManager(new ResourceManager),
 
+  m_connectionManager(new ConnectionManager),
+
+  m_poll(NULL),
+
   m_uploadThrottle(new ThrottleManager),
   m_downloadThrottle(new ThrottleManager),
 
@@ -75,9 +79,9 @@ Manager::Manager() :
   priority_queue_insert(&taskScheduler, &m_taskTick, cachedTime.round_seconds());
 
   m_handshakeManager->slot_connected(rak::make_mem_fun(this, &Manager::receive_connection));
-  m_handshakeManager->slot_download_id(rak::make_mem_fun(this, &Manager::retrive_download_id));
+  m_handshakeManager->slot_download_id(rak::make_mem_fun(m_downloadManager, &DownloadManager::find_info));
 
-  m_listen->slot_incoming(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_incoming));
+  m_connectionManager->listen()->slot_incoming(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_incoming));
 }
 
 Manager::~Manager() {
@@ -90,8 +94,8 @@ Manager::~Manager() {
   delete m_fileManager;
   delete m_handshakeManager;
   delete m_hashQueue;
-  delete m_listen;
   delete m_resourceManager;
+  delete m_connectionManager;
 
   delete m_uploadThrottle;
   delete m_downloadThrottle;
@@ -99,12 +103,13 @@ Manager::~Manager() {
 
 void
 Manager::initialize_download(DownloadWrapper* d) {
-  d->bind_address() = m_bindAddress;
-  d->local_address() = m_localAddress;
+  d->main()->slot_count_handshakes(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::size_info));
+  d->main()->slot_start_handshake(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_outgoing));
 
-  d->set_handshake_manager(m_handshakeManager);
-  d->set_hash_queue(m_hashQueue);
-  d->set_file_manager(m_fileManager);
+  d->hash_checker()->set_queue(m_hashQueue);
+
+  d->main()->content()->entry_list()->slot_insert_filemeta(rak::make_mem_fun(m_fileManager, &FileManager::insert));
+  d->main()->content()->entry_list()->slot_erase_filemeta(rak::make_mem_fun(m_fileManager, &FileManager::erase));
 
   m_downloadManager->insert(d);
   m_resourceManager->insert(d->main(), 1);
@@ -119,6 +124,8 @@ Manager::initialize_download(DownloadWrapper* d) {
 
 void
 Manager::cleanup_download(DownloadWrapper* d) {
+  m_handshakeManager->erase_info(d->info());
+
   m_resourceManager->erase(d->main());
   m_downloadManager->erase(d);
 }
@@ -140,36 +147,25 @@ Manager::receive_tick() {
 }
 
 void
-Manager::receive_connection(SocketFd fd, const std::string& hash, const PeerInfo& peer) {
-  DownloadManager::iterator itr = m_downloadManager->find(hash);
+Manager::receive_connection(SocketFd fd, DownloadInfo* info, const PeerInfo& peer) {
+  DownloadManager::iterator itr = m_downloadManager->find(info);
   
-  if (itr == m_downloadManager->end()) {
-    socketManager.close(fd);
+  if (itr == m_downloadManager->end() ||
+      peer.get_id() == info->local_id() ||
+      !peer.socket_address()->is_valid()) {
+
+    m_connectionManager->dec_socket_count();
+    fd.close();
+    
     return;
   }
 
-  if (!peer.get_socket_address().is_valid()) {
-    (*itr)->main()->signal_network_log().emit("Caught a connection with invalid socket address.");
-
-    socketManager.close(fd);
-    return;
-  }    
-
   if (!(*itr)->main()->is_active() ||
       !(*itr)->main()->connection_list()->insert((*itr)->main(), peer, fd)) {
-    socketManager.close(fd);
+    m_connectionManager->dec_socket_count();
+    fd.close();
     return;
   }
 }
 
-std::string
-Manager::retrive_download_id(const std::string& hash) {
-  DownloadManager::iterator itr = m_downloadManager->find(hash);
-
-  return itr != m_downloadManager->end() &&
-    (*itr)->main()->is_active() &&
-    (*itr)->hash_checker()->is_checked() ?
-    (*itr)->get_local_id() : "";
-}
-
 }
