@@ -41,12 +41,12 @@
 #include <string>
 #include <rak/priority_queue_default.h>
 
-#include "net/socket_address.h"
 #include "net/socket_fd.h"
 
 namespace torrent {
 
-class Listen;
+class Poll;
+
 class HashQueue;
 class HandshakeManager;
 class DownloadManager;
@@ -54,7 +54,9 @@ class DownloadWrapper;
 class FileManager;
 class ResourceManager;
 class PeerInfo;
+class ConnectionManager;
 class ThrottleManager;
+class DownloadInfo;
 
 typedef std::list<std::string> EncodingList;
 
@@ -67,11 +69,12 @@ public:
   FileManager*        file_manager()                            { return m_fileManager; }
   HandshakeManager*   handshake_manager()                       { return m_handshakeManager; }
   HashQueue*          hash_queue()                              { return m_hashQueue; }
-  Listen*             listen()                                  { return m_listen; }
   ResourceManager*    resource_manager()                        { return m_resourceManager; }
 
-  SocketAddress*      local_address()                           { return &m_localAddress; }
-  SocketAddress*      bind_address()                            { return &m_bindAddress; }
+  ConnectionManager*  connection_manager()                      { return m_connectionManager; }
+  
+  Poll*               poll()                                    { return m_poll; }
+  void                set_poll(Poll* p)                         { m_poll = p; }
 
   EncodingList*       encoding_list()                           { return &m_encodingList; }
 
@@ -82,21 +85,18 @@ public:
   void                cleanup_download(DownloadWrapper* d);
 
   void                receive_tick();
-  void                receive_connection(SocketFd fd, const std::string& hash, const PeerInfo& peer);
-
-  std::string         retrive_download_id(const std::string& hash);
+  void                receive_connection(SocketFd fd, DownloadInfo* info, const PeerInfo& peer);
 
 private:
-  SocketAddress       m_localAddress;
-  SocketAddress       m_bindAddress;
-
   DownloadManager*    m_downloadManager;
   FileManager*        m_fileManager;
   HandshakeManager*   m_handshakeManager;
   HashQueue*          m_hashQueue;
-  Listen*             m_listen;
   ResourceManager*    m_resourceManager;
 
+  ConnectionManager*  m_connectionManager;
+  Poll*               m_poll;
+
   EncodingList        m_encodingList;
 
   ThrottleManager*    m_uploadThrottle;
