@@ -36,15 +36,20 @@
 
 #include "config.h"
 
+#include <sstream>
 #include <rak/error_number.h>
 
 #include "torrent/exceptions.h"
 #include "data/chunk_list.h"
 #include "download/choke_manager.h"
 #include "download/chunk_selector.h"
+#include "download/chunk_statistics.h"
 #include "download/download_main.h"
 #include "net/socket_base.h"
 
+#include "torrent/connection_manager.h"
+#include "../manager.h"
+
 #include "peer_connection_base.h"
 
 namespace torrent {
@@ -55,17 +60,10 @@ PeerConnectionBase::PeerConnectionBase() :
   m_down(new ProtocolRead()),
   m_up(new ProtocolWrite()),
 
-  m_peerRate(600),
-
-  m_downThrottle(NULL),
   m_downStall(0),
 
-  m_upThrottle(NULL),
-
   m_sendChoked(false),
-  m_sendInterested(false),
-
-  m_snubbed(false) {
+  m_sendInterested(false) {
 }
 
 PeerConnectionBase::~PeerConnectionBase() {
@@ -76,34 +74,33 @@ PeerConnectionBase::~PeerConnectionBase() {
     throw internal_error("PeerConnection::~PeerConnection() m_fd is valid but m_state and/or m_net is NULL");
 
   m_download->choke_manager()->disconnected(this);
+  m_download->chunk_statistics()->received_disconnect(&m_peerChunks);
 
-  pollCustom->remove_read(this);
-  pollCustom->remove_write(this);
-  pollCustom->remove_error(this);
-  pollCustom->close(this);
+  manager->poll()->remove_read(this);
+  manager->poll()->remove_write(this);
+  manager->poll()->remove_error(this);
+  manager->poll()->close(this);
   
-  socketManager.close(get_fd());
+  manager->connection_manager()->dec_socket_count();
+
+  get_fd().close();
   get_fd().clear();
 
-  if (m_requestList.is_downloading())
-    m_requestList.skip();
+  if (download_queue()->is_downloading())
+    download_queue()->skip();
 
   up_chunk_release();
   down_chunk_release();
 
-  m_requestList.cancel();
-  m_download->chunk_selector()->erase_peer_chunks(&m_peerChunks);
+  download_queue()->cancel();
 
-  m_download->upload_throttle()->erase(m_upThrottle);
-  m_download->download_throttle()->erase(m_downThrottle);
+  m_download->upload_throttle()->erase(m_peerChunks.upload_throttle());
+  m_download->download_throttle()->erase(m_peerChunks.download_throttle());
 
   m_up->set_state(ProtocolWrite::INTERNAL_ERROR);
   m_down->set_state(ProtocolRead::INTERNAL_ERROR);
 
-  delete m_upThrottle;
   delete m_up;
-
-  delete m_downThrottle;
   delete m_down;
 
   m_download = NULL;
@@ -119,29 +116,24 @@ PeerConnectionBase::initialize(DownloadMain* download, const PeerInfo& p, Socket
   m_peer = p;
   m_download = download;
 
-  m_upThrottle = new ThrottleNode(30);
-  m_upThrottle->set_list_iterator(m_download->upload_throttle()->end());
-  m_upThrottle->slot_activate(rak::make_mem_fun(this, &PeerConnectionBase::receive_throttle_up_activate));
+  m_peerChunks.upload_throttle()->set_list_iterator(m_download->upload_throttle()->end());
+  m_peerChunks.upload_throttle()->slot_activate(rak::make_mem_fun(this, &PeerConnectionBase::receive_throttle_up_activate));
 
-  m_downThrottle = new ThrottleNode(30);
-  m_downThrottle->set_list_iterator(m_download->download_throttle()->end());
-  m_downThrottle->slot_activate(rak::make_mem_fun(this, &PeerConnectionBase::receive_throttle_down_activate));
+  m_peerChunks.download_throttle()->set_list_iterator(m_download->download_throttle()->end());
+  m_peerChunks.download_throttle()->slot_activate(rak::make_mem_fun(this, &PeerConnectionBase::receive_throttle_down_activate));
 
-  get_fd().set_throughput();
-
-  m_requestList.set_delegator(m_download->delegator());
-  m_requestList.set_peer_chunks(&m_peerChunks);
+  download_queue()->set_delegator(m_download->delegator());
+  download_queue()->set_peer_chunks(&m_peerChunks);
 
   if (m_download == NULL || !p.is_valid() || !get_fd().is_valid())
     throw internal_error("PeerConnectionBase::set(...) recived bad input.");
 
-  // Set the bitfield size and zero it
-  *m_peerChunks.bitfield() = BitFieldExt(m_download->content()->chunk_total());
+  m_peerChunks.bitfield()->resize(m_download->content()->chunk_total());
 
-  pollCustom->open(this);
-  pollCustom->insert_read(this);
-  pollCustom->insert_write(this);
-  pollCustom->insert_error(this);
+  manager->poll()->open(this);
+  manager->poll()->insert_read(this);
+  manager->poll()->insert_write(this);
+  manager->poll()->insert_error(this);
 
   m_timeLastRead = cachedTime;
 
@@ -155,12 +147,12 @@ PeerConnectionBase::load_down_chunk(const Piece& p) {
   if (!m_download->content()->is_valid_piece(p))
     throw internal_error("Incoming pieces list contains a bad piece");
   
-  if (m_downChunk.is_valid() && p.get_index() == m_downChunk->index())
+  if (m_downChunk.is_valid() && p.index() == m_downChunk->index())
     return;
 
   down_chunk_release();
 
-  m_downChunk = m_download->chunk_list()->get(p.get_index(), true);
+  m_downChunk = m_download->chunk_list()->get(p.index(), true);
   
   if (!m_downChunk.is_valid())
     throw storage_error("File chunk write error: " + std::string(m_downChunk.error_number().c_str()));
@@ -168,12 +160,12 @@ PeerConnectionBase::load_down_chunk(const Piece& p) {
 
 void
 PeerConnectionBase::load_up_chunk() {
-  if (m_upChunk.is_valid() && m_upChunk->index() == m_upPiece.get_index())
+  if (m_upChunk.is_valid() && m_upChunk->index() == m_upPiece.index())
     return;
 
   up_chunk_release();
   
-  m_upChunk = m_download->chunk_list()->get(m_upPiece.get_index(), false);
+  m_upChunk = m_download->chunk_list()->get(m_upPiece.index(), false);
   
   if (!m_upChunk.is_valid())
     throw storage_error("File chunk read error: " + std::string(m_upChunk.error_number().c_str()));
@@ -181,11 +173,11 @@ PeerConnectionBase::load_up_chunk() {
 
 void
 PeerConnectionBase::set_snubbed(bool v) {
-  if (v == m_snubbed)
+  if (v == m_peerChunks.is_snubbed())
     return;
 
   bool wasUploadWanted = is_upload_wanted();
-  m_snubbed = v;
+  m_peerChunks.set_snubbed(v);
 
   if (v) {
     if (wasUploadWanted)
@@ -211,12 +203,12 @@ PeerConnectionBase::receive_choke(bool v) {
 
 void
 PeerConnectionBase::receive_throttle_down_activate() {
-  pollCustom->insert_read(this);
+  manager->poll()->insert_read(this);
 }
 
 void
 PeerConnectionBase::receive_throttle_up_activate() {
-  pollCustom->insert_write(this);
+  manager->poll()->insert_write(this);
 }
 
 void
@@ -226,28 +218,28 @@ PeerConnectionBase::event_error() {
 
 bool
 PeerConnectionBase::down_chunk() {
-  if (!m_download->download_throttle()->is_throttled(m_downThrottle))
+  if (!m_download->download_throttle()->is_throttled(m_peerChunks.download_throttle()))
     throw internal_error("PeerConnectionBase::down_chunk() tried to read a piece but is not in throttle list");
 
   if (!m_downChunk->chunk()->is_writable())
     throw internal_error("PeerConnectionBase::down_part() chunk not writable, permission denided");
 
-  uint32_t quota = m_download->download_throttle()->node_quota(m_downThrottle);
+  uint32_t quota = m_download->download_throttle()->node_quota(m_peerChunks.download_throttle());
 
   if (quota == 0) {
-    pollCustom->remove_read(this);
-    m_download->download_throttle()->node_deactivate(m_downThrottle);
+    manager->poll()->remove_read(this);
+    m_download->download_throttle()->node_deactivate(m_peerChunks.download_throttle());
     return false;
   }
 
   uint32_t count;
-  uint32_t left = quota = std::min(quota, m_downPiece.get_length() - m_down->position());
+  uint32_t left = quota = std::min(quota, m_downPiece.length() - m_down->position());
 
   Chunk::MemoryArea memory;
-  ChunkPart part = m_downChunk->chunk()->at_position(m_downPiece.get_offset() + m_down->position());
+  ChunkPart part = m_downChunk->chunk()->at_position(m_downPiece.offset() + m_down->position());
 
   do {
-    memory = m_downChunk->chunk()->at_memory(m_downPiece.get_offset() + m_down->position(), part++);
+    memory = m_downChunk->chunk()->at_memory(m_downPiece.offset() + m_down->position(), part++);
     count = read_stream_throws(memory.first, std::min(left, memory.second));
 
     m_down->adjust_position(count);
@@ -257,23 +249,23 @@ PeerConnectionBase::down_chunk() {
 
   uint32_t bytes = quota - left;
 
-  m_download->download_throttle()->node_used(m_downThrottle, bytes);
-  m_download->down_rate()->insert(bytes);
+  m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), bytes);
+  m_download->info()->down_rate()->insert(bytes);
 
-  return m_down->position() == m_downPiece.get_length();
+  return m_down->position() == m_downPiece.length();
 }
 
 bool
 PeerConnectionBase::down_chunk_from_buffer() {
   uint32_t count, quota;
   uint32_t left = quota = std::min<uint32_t>(m_down->buffer()->remaining(),
-					     m_downPiece.get_length() - m_down->position());
+					     m_downPiece.length() - m_down->position());
 
   Chunk::MemoryArea memory;
-  ChunkPart part = m_downChunk->chunk()->at_position(m_downPiece.get_offset() + m_down->position());
+  ChunkPart part = m_downChunk->chunk()->at_position(m_downPiece.offset() + m_down->position());
 
   do {
-    memory = m_downChunk->chunk()->at_memory(m_downPiece.get_offset() + m_down->position(), part++);
+    memory = m_downChunk->chunk()->at_memory(m_downPiece.offset() + m_down->position(), part++);
     count = std::min(left, memory.second);
 
     std::memcpy(memory.first, m_down->buffer()->position(), count);
@@ -286,36 +278,36 @@ PeerConnectionBase::down_chunk_from_buffer() {
 
   uint32_t bytes = quota - left;
 
-  m_download->download_throttle()->node_used(m_downThrottle, bytes);
-  m_download->down_rate()->insert(bytes);
+  m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), bytes);
+  m_download->info()->down_rate()->insert(bytes);
 
-  return m_down->position() == m_downPiece.get_length();
+  return m_down->position() == m_downPiece.length();
 }
 
 bool
 PeerConnectionBase::up_chunk() {
-  if (!m_download->upload_throttle()->is_throttled(m_upThrottle))
+  if (!m_download->upload_throttle()->is_throttled(m_peerChunks.upload_throttle()))
     throw internal_error("PeerConnectionBase::up_chunk() tried to write a piece but is not in throttle list");
 
   if (!m_upChunk->chunk()->is_readable())
     throw internal_error("ProtocolChunk::write_part() chunk not readable, permission denided");
 
-  uint32_t quota = m_download->upload_throttle()->node_quota(m_upThrottle);
+  uint32_t quota = m_download->upload_throttle()->node_quota(m_peerChunks.upload_throttle());
 
   if (quota == 0) {
-    pollCustom->remove_write(this);
-    m_download->upload_throttle()->node_deactivate(m_upThrottle);
+    manager->poll()->remove_write(this);
+    m_download->upload_throttle()->node_deactivate(m_peerChunks.upload_throttle());
     return false;
   }
 
   uint32_t count;
-  uint32_t left = quota = std::min(quota, m_upPiece.get_length() - m_up->position());
+  uint32_t left = quota = std::min(quota, m_upPiece.length() - m_up->position());
 
   Chunk::MemoryArea memory;
-  ChunkPart part = m_upChunk->chunk()->at_position(m_upPiece.get_offset() + m_up->position());
+  ChunkPart part = m_upChunk->chunk()->at_position(m_upPiece.offset() + m_up->position());
 
   do {
-    memory = m_upChunk->chunk()->at_memory(m_upPiece.get_offset() + m_up->position(), part++);
+    memory = m_upChunk->chunk()->at_memory(m_upPiece.offset() + m_up->position(), part++);
     count = write_stream_throws(memory.first, std::min(left, memory.second));
 
     m_up->adjust_position(count);
@@ -325,10 +317,10 @@ PeerConnectionBase::up_chunk() {
 
   uint32_t bytes = quota - left;
 
-  m_download->upload_throttle()->node_used(m_upThrottle, bytes);
-  m_download->up_rate()->insert(bytes);
+  m_download->upload_throttle()->node_used(m_peerChunks.upload_throttle(), bytes);
+  m_download->info()->up_rate()->insert(bytes);
 
-  return m_up->position() == m_upPiece.get_length();
+  return m_up->position() == m_upPiece.length();
 }
 
 void
@@ -349,21 +341,21 @@ PeerConnectionBase::up_chunk_release() {
 
 void
 PeerConnectionBase::read_request_piece(const Piece& p) {
-  PieceList::iterator itr = std::find(m_sendList.begin(), m_sendList.end(), p);
+  PeerChunks::piece_list_type::iterator itr = std::find(m_peerChunks.upload_queue()->begin(), m_peerChunks.upload_queue()->end(), p);
   
-  if (m_up->choked() || itr != m_sendList.end() || p.get_length() > (1 << 17))
+  if (m_up->choked() || itr != m_peerChunks.upload_queue()->end() || p.length() > (1 << 17))
     return;
 
-  m_sendList.push_back(p);
+  m_peerChunks.upload_queue()->push_back(p);
   write_insert_poll_safe();
 }
 
 void
 PeerConnectionBase::read_cancel_piece(const Piece& p) {
-  PieceList::iterator itr = std::find(m_sendList.begin(), m_sendList.end(), p);
+  PeerChunks::piece_list_type::iterator itr = std::find(m_peerChunks.upload_queue()->begin(), m_peerChunks.upload_queue()->end(), p);
   
-  if (itr != m_sendList.end())
-    m_sendList.erase(itr);
+  if (itr != m_peerChunks.upload_queue()->end())
+    m_peerChunks.upload_queue()->erase(itr);
 }  
 
 void
@@ -378,20 +370,21 @@ PeerConnectionBase::read_buffer_move_unused() {
 
 void
 PeerConnectionBase::write_prepare_piece() {
-  m_upPiece = m_sendList.front();
-  m_sendList.pop_front();
+  m_upPiece = m_peerChunks.upload_queue()->front();
+  m_peerChunks.upload_queue()->pop_front();
 
   // Move these checks somewhere else?
   if (!m_download->content()->is_valid_piece(m_upPiece) ||
-      !m_download->content()->has_chunk(m_upPiece.get_index())) {
-//     std::stringstream s;
+      !m_download->content()->has_chunk(m_upPiece.index())) {
+    std::stringstream s;
 
-//     s << "Peer requested a piece with invalid index or length/offset: "
-//       << m_upPiece.get_index() << ' '
-//       << m_upPiece.get_length() << ' '
-//       << m_upPiece.get_offset();
+    s << "Peer requested a piece with invalid index or length/offset: "
+      << m_upPiece.index() << ' '
+      << m_upPiece.length() << ' '
+      << m_upPiece.offset();
 
-    throw communication_error("Peer requested a piece with invalid index or length/offset.");
+    throw communication_error(s.str());
+//     throw communication_error("Peer requested a piece with invalid index or length/offset.");
   }
       
   m_up->write_piece(m_upPiece);
@@ -426,8 +419,8 @@ PeerConnectionBase::read_bitfield_from_buffer(uint32_t msgLength) {
 
 bool
 PeerConnectionBase::write_bitfield_body() {
-  m_up->adjust_position(write_stream_throws(m_download->content()->bitfield().begin() + m_up->position(),
-					    m_download->content()->bitfield().size_bytes() - m_up->position()));
+  m_up->adjust_position(write_stream_throws(m_download->content()->bitfield()->begin() + m_up->position(),
+					    m_download->content()->bitfield()->size_bytes() - m_up->position()));
 
   return m_up->position() == m_peerChunks.bitfield()->size_bytes();
 }
@@ -448,24 +441,24 @@ PeerConnectionBase::should_request() {
     // We check if the peer is stalled, if it is not then we should
     // request. If the peer is stalled then we only request if the
     // download rate is below a certain value.
-    return m_downStall <= 1 || m_download->down_rate()->rate() < (10 << 10);
+    return m_downStall <= 1 || m_download->info()->down_rate()->rate() < (10 << 10);
 }
 
 bool
 PeerConnectionBase::try_request_pieces() {
-  if (m_requestList.empty())
+  if (download_queue()->empty())
     m_downStall = 0;
 
-  uint32_t pipeSize = m_requestList.calculate_pipe_size(m_downThrottle->rate()->rate());
+  uint32_t pipeSize = download_queue()->calculate_pipe_size(m_peerChunks.download_throttle()->rate()->rate());
   bool success = false;
 
-  while (m_requestList.size() < pipeSize && m_up->can_write_request()) {
-    const Piece* p = m_requestList.delegate();
+  while (download_queue()->size() < pipeSize && m_up->can_write_request()) {
+    const Piece* p = download_queue()->delegate();
 
     if (p == NULL)
       break;
 
-    if (!m_download->content()->is_valid_piece(*p) || !m_peerChunks.bitfield()->get(p->get_index()))
+    if (!m_download->content()->is_valid_piece(*p) || !m_peerChunks.bitfield()->get(p->index()))
       throw internal_error("PeerConnectionBase::try_request_pieces() tried to use an invalid piece.");
 
     m_up->write_request(*p);
@@ -478,7 +471,7 @@ PeerConnectionBase::try_request_pieces() {
 
 void
 PeerConnectionBase::set_remote_interested() {
-  if (m_down->interested() || m_peerChunks.bitfield()->all_set())
+  if (m_down->interested() || m_peerChunks.bitfield()->is_all_set())
     return;
 
   m_down->set_interested(true);
