@@ -42,6 +42,7 @@
 #include "data/content.h"
 #include "data/chunk_list_node.h"
 #include "download/chunk_selector.h"
+#include "download/chunk_statistics.h"
 #include "download/download_main.h"
 
 #include "peer_connection_leech.h"
@@ -58,7 +59,7 @@ PeerConnectionLeech::~PeerConnectionLeech() {
 void
 PeerConnectionLeech::initialize_custom() {
   if (m_download->content()->chunks_completed() != 0) {
-    m_up->write_bitfield(m_download->content()->bitfield().size_bytes());
+    m_up->write_bitfield(m_download->content()->bitfield()->size_bytes());
 
     m_up->buffer()->prepare_end();
     m_up->set_position(0);
@@ -69,6 +70,7 @@ PeerConnectionLeech::initialize_custom() {
 void
 PeerConnectionLeech::update_interested() {
 // FIXME:   if (m_download->delegator()->get_select().interested(m_peerChunks.bitfield()->bitfield())) {
+
   if (true) {
     m_sendInterested = !m_up->interested();
     m_up->set_interested(true);
@@ -76,15 +78,17 @@ PeerConnectionLeech::update_interested() {
     m_sendInterested = m_up->interested();
     m_up->set_interested(false);
   }
+
+  m_peerChunks.download_cache()->clear();
 }
 
 // Disconnecting connections where both are seeders should be done by
 // DownloadMain when it finishes the last chunk.
 void
 PeerConnectionLeech::receive_finished_chunk(int32_t index) {
-  m_haveQueue.push_back(index);
+  m_peerChunks.have_queue()->push_back(index);
 
-  if (m_requestList.has_index(index))
+  if (download_queue()->has_index(index))
     throw internal_error("PeerConnection::sendHave(...) found a request with the same index");
 }
 
@@ -111,8 +115,8 @@ PeerConnectionLeech::receive_keepalive() {
   // should stay at zero or one when downloading at an acceptable
   // speed. Thus only when m_downStall >= 2 is the download actually
   // stalling.
-  if (!m_requestList.empty() && m_downStall++ > 0)
-    m_requestList.stall();
+  if (!download_queue()->empty() && m_downStall++ > 0)
+    download_queue()->stall();
 
   return true;
 }
@@ -163,8 +167,10 @@ PeerConnectionLeech::read_message() {
   case ProtocolBase::CHOKE:
     m_down->set_choked(true);
 
-    m_requestList.cancel();
-    m_download->download_throttle()->erase(m_downThrottle);
+    m_peerChunks.download_cache()->disable();
+
+    download_queue()->cancel();
+    m_download->download_throttle()->erase(m_peerChunks.download_throttle());
 
     return true;
 
@@ -187,13 +193,18 @@ PeerConnectionLeech::read_message() {
     return true;
 
   case ProtocolBase::HAVE:
-    if (buf->remaining() < 4)
+    if (!m_down->can_read_have_body())
       break;
 
     read_have_chunk(buf->read_32());
     return true;
 
   case ProtocolBase::BITFIELD:
+    // Bad peer, sending their bitfield after other messages have been
+    // sent.
+    if (m_peerChunks.using_counter() || !m_peerChunks.bitfield()->is_all_unset())
+      throw close_connection();
+
     if (read_bitfield_from_buffer(length - 1)) {
       finish_bitfield();
       return true;
@@ -204,7 +215,7 @@ PeerConnectionLeech::read_message() {
     }
 
   case ProtocolBase::REQUEST:
-    if (buf->remaining() < 13)
+    if (!m_down->can_read_request_body())
       break;
 
     if (!m_up->choked()) {
@@ -218,7 +229,7 @@ PeerConnectionLeech::read_message() {
     return true;
 
   case ProtocolBase::PIECE:
-    if (buf->remaining() < 9)
+    if (!m_down->can_read_piece_body())
       break;
 
     m_down->set_position(0);
@@ -227,10 +238,10 @@ PeerConnectionLeech::read_message() {
     if (!receive_piece_header()) {
 
       // We're skipping this piece.
-      m_down->set_position(std::min<uint32_t>(buf->remaining(), m_downPiece.get_length()));
+      m_down->set_position(std::min<uint32_t>(buf->remaining(), m_downPiece.length()));
       buf->move_position(m_down->position());
 
-      if (m_down->position() != m_downPiece.get_length()) {
+      if (m_down->position() != m_downPiece.length()) {
 	m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
 	return false;
 
@@ -241,7 +252,7 @@ PeerConnectionLeech::read_message() {
     } else if (down_chunk_from_buffer()) {
       // Done with chunk.
       m_downChunk->set_time_modified(cachedTime);
-      m_requestList.finished();
+      download_queue()->finished();
     
       if (m_downStall > 0)
 	m_downStall--;
@@ -255,12 +266,12 @@ PeerConnectionLeech::read_message() {
 
     } else {
       m_down->set_state(ProtocolRead::READ_PIECE);
-      m_download->download_throttle()->insert(m_downThrottle);
+      m_download->download_throttle()->insert(m_peerChunks.download_throttle());
       return false;
     }
 
   case ProtocolBase::CANCEL:
-    if (buf->remaining() < 13)
+    if (!m_down->can_read_cancel_body())
       break;
 
     read_cancel_piece(m_down->read_request());
@@ -312,12 +323,12 @@ PeerConnectionLeech::event_read() {
 	}
 
       case ProtocolRead::READ_PIECE:
-	if (!m_requestList.is_downloading())
+	if (!download_queue()->is_downloading())
 	  throw internal_error("ProtocolRead::READ_PIECE state but RequestList is not downloading");
 
-	if (!m_requestList.is_wanted()) {
+	if (!download_queue()->is_wanted()) {
 	  m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
-	  m_requestList.skip();
+	  download_queue()->skip();
 
 	  break;
 	}
@@ -326,7 +337,7 @@ PeerConnectionLeech::event_read() {
 	  return;
 
 	m_downChunk->set_time_modified(cachedTime);
-	m_requestList.finished();
+	download_queue()->finished();
 	
 	if (m_downStall > 0)
 	  m_downStall--;
@@ -340,9 +351,9 @@ PeerConnectionLeech::event_read() {
 	break;
 
       case ProtocolRead::READ_SKIP_PIECE:
-	m_down->adjust_position(ignore_stream_throws(m_downPiece.get_length() - m_down->position()));
+	m_down->adjust_position(ignore_stream_throws(m_downPiece.length() - m_down->position()));
 
-	if (m_down->position() != m_downPiece.get_length())
+	if (m_down->position() != m_downPiece.length())
 	  return;
 
 	m_down->set_state(ProtocolRead::IDLE);
@@ -373,16 +384,16 @@ PeerConnectionLeech::event_read() {
     m_download->connection_list()->erase(this);
 
   } catch (blocked_connection& e) {
-    m_download->signal_network_log().emit("Momentarily blocked read connection.");
+    m_download->info()->signal_network_log().emit("Momentarily blocked read connection.");
     m_download->connection_list()->erase(this);
 
   } catch (network_error& e) {
-    m_download->signal_network_log().emit(e.what());
+    m_download->info()->signal_network_log().emit(e.what());
 
     m_download->connection_list()->erase(this);
 
   } catch (storage_error& e) {
-    m_download->signal_storage_error().emit(e.what());
+    m_download->info()->signal_storage_error().emit(e.what());
     m_download->connection_list()->erase(this);
 
   } catch (base_error& e) {
@@ -403,12 +414,12 @@ PeerConnectionLeech::fill_write_buffer() {
     m_up->write_choke(m_up->choked());
 
     if (m_up->choked()) {
-      m_download->upload_throttle()->erase(m_upThrottle);
+      m_download->upload_throttle()->erase(m_peerChunks.upload_throttle());
       up_chunk_release();
-      m_sendList.clear();
+      m_peerChunks.upload_queue()->clear();
 
     } else {
-      m_download->upload_throttle()->insert(m_upThrottle);
+      m_download->upload_throttle()->insert(m_peerChunks.upload_throttle());
     }
   }
 
@@ -426,18 +437,18 @@ PeerConnectionLeech::fill_write_buffer() {
       !(m_tryRequest = !should_request()) &&
       !(m_tryRequest = try_request_pieces()) &&
 
-      !m_requestList.is_interested_in_active()) {
-//     m_sendInterested = true;
-//     m_up->set_interested(false);
+      !download_queue()->is_interested_in_active()) {
+    m_sendInterested = true;
+    m_up->set_interested(false);
   }
 
-  while (!m_haveQueue.empty() && m_up->can_write_have()) {
-    m_up->write_have(m_haveQueue.front());
-    m_haveQueue.pop_front();
+  while (!m_peerChunks.have_queue()->empty() && m_up->can_write_have()) {
+    m_up->write_have(m_peerChunks.have_queue()->front());
+    m_peerChunks.have_queue()->pop_front();
   }
 
   if (!m_up->choked() &&
-      !m_sendList.empty() &&
+      !m_peerChunks.upload_queue()->empty() &&
       m_up->can_write_piece())
     write_prepare_piece();
 }
@@ -460,7 +471,7 @@ PeerConnectionLeech::event_write() {
 	fill_write_buffer();
 
 	if (m_up->buffer()->size_position() == 0) {
-	  pollCustom->remove_write(this);
+	  manager->poll()->remove_write(this);
 	  return;
 	}
 
@@ -522,15 +533,15 @@ PeerConnectionLeech::event_write() {
     m_download->connection_list()->erase(this);
 
   } catch (blocked_connection& e) {
-    m_download->signal_network_log().emit("Momentarily blocked write connection.");
+    m_download->info()->signal_network_log().emit("Momentarily blocked write connection.");
     m_download->connection_list()->erase(this);
 
   } catch (network_error& e) {
-    m_download->signal_network_log().emit(e.what());
+    m_download->info()->signal_network_log().emit(e.what());
     m_download->connection_list()->erase(this);
 
   } catch (storage_error& e) {
-    m_download->signal_storage_error().emit(e.what());
+    m_download->info()->signal_storage_error().emit(e.what());
     m_download->connection_list()->erase(this);
 
   } catch (base_error& e) {
@@ -551,10 +562,9 @@ PeerConnectionLeech::read_have_chunk(uint32_t index) {
   if (m_peerChunks.bitfield()->get(index))
     return;
 
-  m_peerChunks.bitfield()->set(index, true);
-  m_peerRate.insert(m_download->content()->chunk_size());
+  m_download->chunk_statistics()->received_have_chunk(&m_peerChunks, index, m_download->content()->chunk_size());
 
-  if (m_peerChunks.bitfield()->all_set())
+  if (m_peerChunks.bitfield()->is_all_set())
     if (m_download->content()->is_done())
       throw close_connection();
     else
@@ -565,21 +575,21 @@ PeerConnectionLeech::read_have_chunk(uint32_t index) {
 
   if (is_up_interested()) {
 
-    if (!m_tryRequest && true) { // FIXME: m_download->delegator()->get_select().interested(index)) {
+    if (!m_tryRequest && m_download->chunk_selector()->received_have_chunk(&m_peerChunks, index)) {
       m_tryRequest = true;
       write_insert_poll_safe();
     }
 
   } else {
 
-// FIXME:    if (m_download->delegator()->get_select().interested(index)) {
-    if (true) {
+    if (m_download->chunk_selector()->received_have_chunk(&m_peerChunks, index)) {
       m_sendInterested = true;
       m_up->set_interested(true);
       
       // Is it enough to insert into write here? Make the interested
       // check branch to include insert_write, even when not sending
       // interested.
+      m_tryRequest = true;
       write_insert_poll_safe();
     }
   }
@@ -587,14 +597,16 @@ PeerConnectionLeech::read_have_chunk(uint32_t index) {
 
 void
 PeerConnectionLeech::finish_bitfield() {
-  m_peerChunks.bitfield()->update_count();
+  m_peerChunks.bitfield()->update();
 
-  if (m_download->content()->is_done() && m_peerChunks.bitfield()->all_set())
+  if (m_download->content()->is_done() && m_peerChunks.bitfield()->is_all_set())
     throw close_connection();
 
-  m_download->chunk_selector()->insert_peer_chunks(&m_peerChunks);
+  m_download->chunk_statistics()->received_connect(&m_peerChunks);
 
-  if (!m_download->content()->is_done() && true) { // FIXME: m_download->delegator()->get_select().interested(m_peerChunks.bitfield()->bitfield())) {
+  // By default, set interested. When unchoked we'll check if there's
+  // any interesting pieces, then set uninterested.
+  if (!m_download->content()->is_done()) {
     m_sendInterested = true;
     m_up->set_interested(true);
   }
@@ -604,14 +616,14 @@ PeerConnectionLeech::finish_bitfield() {
 
 bool
 PeerConnectionLeech::receive_piece_header() {
-  if (m_requestList.downloading(m_downPiece)) {
+  if (download_queue()->downloading(m_downPiece)) {
 
     load_down_chunk(m_downPiece);
     return true;
 
   } else {
-    if (m_downPiece.get_length() == 0)
-      m_download->signal_network_log().emit("Received piece with length zero");
+    if (m_downPiece.length() == 0)
+      m_download->info()->signal_network_log().emit("Received piece with length zero");
 
     return false;
   }
