@@ -40,6 +40,7 @@
 #include <sstream>
 
 #include "data/content.h"
+#include "download/chunk_statistics.h"
 #include "download/download_main.h"
 
 #include "peer_connection_seed.h"
@@ -56,7 +57,7 @@ PeerConnectionSeed::~PeerConnectionSeed() {
 void
 PeerConnectionSeed::initialize_custom() {
   if (m_download->content()->chunks_completed() != 0) {
-    m_up->write_bitfield(m_download->content()->bitfield().size_bytes());
+    m_up->write_bitfield(m_download->content()->bitfield()->size_bytes());
 
     m_up->buffer()->prepare_end();
     m_up->set_position(0);
@@ -151,13 +152,18 @@ PeerConnectionSeed::read_message() {
     return true;
 
   case ProtocolBase::HAVE:
-    if (buf->remaining() < 4)
+    if (!m_down->can_read_have_body())
       break;
 
     read_have_chunk(buf->read_32());
     return true;
 
   case ProtocolBase::BITFIELD:
+    // Bad peer, sending their bitfield after other messages have been
+    // sent.
+    if (m_peerChunks.using_counter() || !m_peerChunks.bitfield()->is_all_unset())
+      throw close_connection();
+
     if (read_bitfield_from_buffer(length - 1)) {
       finish_bitfield();
       return true;
@@ -168,7 +174,7 @@ PeerConnectionSeed::read_message() {
     }
 
   case ProtocolBase::REQUEST:
-    if (buf->remaining() < 13)
+    if (!m_down->can_read_request_body())
       break;
 
     if (!m_up->choked()) {
@@ -185,7 +191,7 @@ PeerConnectionSeed::read_message() {
     throw network_error("Received a piece but the connection is strictly for seeding.");
 
   case ProtocolBase::CANCEL:
-    if (buf->remaining() < 13)
+    if (!m_down->can_read_cancel_body())
       break;
 
     read_cancel_piece(m_down->read_request());
@@ -259,16 +265,16 @@ PeerConnectionSeed::event_read() {
     m_download->connection_list()->erase(this);
 
   } catch (blocked_connection& e) {
-    m_download->signal_network_log().emit("Momentarily blocked read connection.");
+    m_download->info()->signal_network_log().emit("Momentarily blocked read connection.");
     m_download->connection_list()->erase(this);
 
   } catch (network_error& e) {
-    m_download->signal_network_log().emit(e.what());
+    m_download->info()->signal_network_log().emit(e.what());
 
     m_download->connection_list()->erase(this);
 
   } catch (storage_error& e) {
-    m_download->signal_storage_error().emit(e.what());
+    m_download->info()->signal_storage_error().emit(e.what());
     m_download->connection_list()->erase(this);
 
   } catch (base_error& e) {
@@ -289,17 +295,17 @@ PeerConnectionSeed::fill_write_buffer() {
     m_up->write_choke(m_up->choked());
 
     if (m_up->choked()) {
-      m_download->upload_throttle()->erase(m_upThrottle);
+      m_download->upload_throttle()->erase(m_peerChunks.upload_throttle());
       up_chunk_release();
-      m_sendList.clear();
+      m_peerChunks.upload_queue()->clear();
 
     } else {
-      m_download->upload_throttle()->insert(m_upThrottle);
+      m_download->upload_throttle()->insert(m_peerChunks.upload_throttle());
     }
   }
 
   if (!m_up->choked() &&
-      !m_sendList.empty() &&
+      !m_peerChunks.upload_queue()->empty() &&
       m_up->can_write_piece())
     write_prepare_piece();
 }
@@ -322,7 +328,7 @@ PeerConnectionSeed::event_write() {
 	fill_write_buffer();
 
 	if (m_up->buffer()->size_position() == 0) {
-	  pollCustom->remove_write(this);
+	  manager->poll()->remove_write(this);
 	  return;
 	}
 
@@ -384,15 +390,15 @@ PeerConnectionSeed::event_write() {
     m_download->connection_list()->erase(this);
 
   } catch (blocked_connection& e) {
-    m_download->signal_network_log().emit("Momentarily blocked write connection.");
+    m_download->info()->signal_network_log().emit("Momentarily blocked write connection.");
     m_download->connection_list()->erase(this);
 
   } catch (network_error& e) {
-    m_download->signal_network_log().emit(e.what());
+    m_download->info()->signal_network_log().emit(e.what());
     m_download->connection_list()->erase(this);
 
   } catch (storage_error& e) {
-    m_download->signal_storage_error().emit(e.what());
+    m_download->info()->signal_storage_error().emit(e.what());
     m_download->connection_list()->erase(this);
 
   } catch (base_error& e) {
@@ -410,21 +416,24 @@ PeerConnectionSeed::read_have_chunk(uint32_t index) {
   if (index >= m_peerChunks.bitfield()->size_bits())
     throw network_error("Peer sent HAVE message with out-of-range index.");
 
-  m_peerChunks.bitfield()->set(index, true);
-  m_peerRate.insert(m_download->content()->chunk_size());
+  if (m_peerChunks.bitfield()->get(index))
+    return;
 
-  if (m_peerChunks.bitfield()->all_set())
+  m_download->chunk_statistics()->received_have_chunk(&m_peerChunks, index, m_download->content()->chunk_size());
+  //m_download->chunk_selector()->received_have_chunk(&m_peerChunks, index);
+
+  if (m_peerChunks.bitfield()->is_all_set())
     throw close_connection();
 }
 
 void
 PeerConnectionSeed::finish_bitfield() {
-  m_peerChunks.bitfield()->update_count();
+  m_peerChunks.bitfield()->update();
 
-  if (m_peerChunks.bitfield()->all_set())
+  if (m_peerChunks.bitfield()->is_all_set())
     throw close_connection();
 
-//   m_download->bitfield_counter().inc(m_peerChunks.bitfield()->bitfield());
+  m_download->chunk_statistics()->received_connect(&m_peerChunks);
 
   write_insert_poll_safe();
 }
