@@ -36,23 +36,27 @@
 
 #include "config.h"
 
-#include "exceptions.h"
-#include "download.h"
+#include <rak/functional.h>
+#include <sigc++/bind.h>
+#include <sigc++/hide.h>
 
 #include "data/hash_queue.h"
 #include "data/hash_torrent.h"
 #include "download/choke_manager.h"
 #include "download/chunk_selector.h"
+#include "download/chunk_statistics.h"
 #include "download/delegator_chunk.h"
 #include "download/download_wrapper.h"
 #include "protocol/peer_connection_base.h"
 #include "protocol/peer_factory.h"
-#include "tracker/tracker_info.h"
+#include "download/download_info.h"
 #include "tracker/tracker_manager.h"
 
-#include <rak/functional.h>
-#include <sigc++/bind.h>
-#include <sigc++/hide.h>
+#include "exceptions.h"
+#include "download.h"
+#include "file_list.h"
+#include "object.h"
+#include "tracker_list.h"
 
 namespace torrent {
 
@@ -76,6 +80,9 @@ Download::start() {
 
 void
 Download::stop() {
+  if (!m_ptr->main()->is_active())
+    return;
+
   m_ptr->stop();
 }
 
@@ -98,7 +105,8 @@ Download::hash_resume_save() {
 
 void
 Download::hash_resume_clear() {
-  m_ptr->bencode().erase_key("libtorrent resume");
+  if (m_ptr->bencode()->has_key("libtorrent resume"))
+    m_ptr->bencode()->get_key("libtorrent resume").erase_key("bitfield");
 }
 
 bool
@@ -111,14 +119,9 @@ Download::is_active() const {
   return m_ptr->main()->is_active();
 }
 
-bool
-Download::is_tracker_busy() const {
-  return m_ptr->main()->tracker_manager()->is_busy();
-}
-
 bool
 Download::is_hash_checked() const {
-  return m_ptr->hash_checker()->is_checked();
+  return m_ptr->main()->is_open() && m_ptr->hash_checker()->is_checked();
 }
 
 bool
@@ -126,60 +129,76 @@ Download::is_hash_checking() const {
   return m_ptr->hash_checker()->is_checking();
 }
 
-std::string
+const std::string&
 Download::name() const {
-  return m_ptr ? m_ptr->get_name() : "";
+  if (m_ptr == NULL)
+    throw internal_error("Download::name() m_ptr == NULL.");
+
+  return m_ptr->info()->name();
 }
 
-std::string
+const std::string&
 Download::info_hash() const {
-  return m_ptr ? m_ptr->get_hash() : "";
+  if (m_ptr == NULL)
+    throw internal_error("Download::info_hash() m_ptr == NULL.");
+
+  return m_ptr->info()->hash();
 }
 
-std::string
+const std::string&
 Download::local_id() const {
-  return m_ptr ? m_ptr->get_local_id() : "";
+  if (m_ptr == NULL)
+    throw internal_error("Download::local_id() m_ptr == NULL.");
+
+  return m_ptr->info()->local_id();
 }
 
 uint32_t
 Download::creation_date() const {
-  if (m_ptr->bencode().has_key("creation date") && m_ptr->bencode().get_key("creation date").is_value())
-    return m_ptr->bencode().get_key("creation date").as_value();
+  if (m_ptr->bencode()->has_key_value("creation date"))
+    return m_ptr->bencode()->get_key("creation date").as_value();
   else
     return 0;
 }
 
-Bencode&
+Object*
 Download::bencode() {
   return m_ptr->bencode();
 }
 
-const Bencode&
+const Object*
 Download::bencode() const {
   return m_ptr->bencode();
 }
 
-std::string
-Download::root_dir() const {
-  return m_ptr->main()->content()->root_dir();
+FileList
+Download::file_list() const {
+  return FileList(m_ptr->main()->content()->entry_list());
 }
 
-void
-Download::set_root_dir(const std::string& dir) {
-  if (is_open())
-    throw input_error("Tried to change the root directory for an open download.");
+TrackerList
+Download::tracker_list() const {
+  return TrackerList(m_ptr->main()->tracker_manager());
+}
 
-  m_ptr->main()->content()->set_root_dir(dir);
+Rate*
+Download::down_rate() {
+  return m_ptr->info()->down_rate();
 }
 
 const Rate*
 Download::down_rate() const {
-  return m_ptr->main()->down_rate();
+  return m_ptr->info()->down_rate();
+}
+
+Rate*
+Download::up_rate() {
+  return m_ptr->info()->up_rate();
 }
 
 const Rate*
 Download::up_rate() const {
-  return m_ptr->main()->up_rate();
+  return m_ptr->info()->up_rate();
 }
 
 uint64_t
@@ -191,7 +210,7 @@ Download::bytes_done() const {
   for (Delegator::Chunks::iterator itr1 = d->get_chunks().begin(), last1 = d->get_chunks().end(); itr1 != last1; ++itr1)
     for (DelegatorChunk::iterator itr2 = (*itr1)->begin(), last2 = (*itr1)->end(); itr2 != last2; ++itr2)
       if (itr2->is_finished())
-	a += itr2->get_piece().get_length();
+	a += itr2->get_piece().length();
   
   return a + m_ptr->main()->content()->bytes_completed();
 }
@@ -221,14 +240,14 @@ Download::chunks_hashed() const {
   return m_ptr->hash_checker()->position();
 }
 
-const unsigned char*
-Download::bitfield_data() const {
-  return (unsigned char*)m_ptr->main()->content()->bitfield().begin();
+const uint8_t*
+Download::chunks_seen() const {
+  return !m_ptr->main()->chunk_statistics()->empty() ? &*m_ptr->main()->chunk_statistics()->begin() : NULL;
 }
 
-uint32_t
-Download::bitfield_size() const {
-  return m_ptr->main()->content()->bitfield().size_bits();
+const Bitfield*
+Download::bitfield() const {
+  return m_ptr->main()->content()->bitfield();
 }
 
 uint32_t
@@ -251,6 +270,16 @@ Download::peers_not_connected() const {
   return m_ptr->main()->available_list()->size();
 }
 
+uint32_t
+Download::peers_complete() const {
+  return m_ptr->main()->chunk_statistics()->complete();
+}
+
+uint32_t
+Download::peers_accounted() const {
+  return m_ptr->main()->chunk_statistics()->accounted();
+}
+
 uint32_t
 Download::peers_currently_unchoked() const {
   return m_ptr->main()->choke_manager()->currently_unchoked();
@@ -261,21 +290,16 @@ Download::peers_currently_interested() const {
   return m_ptr->main()->choke_manager()->currently_interested();
 }
 
+bool
+Download::accepting_new_peers() const {
+  return m_ptr->info()->accepting_new_peers();
+}
+
 uint32_t
 Download::uploads_max() const {
   return m_ptr->main()->choke_manager()->max_unchoked();
 }
   
-uint64_t
-Download::tracker_timeout() const {
-  return std::max(m_ptr->main()->tracker_manager()->get_next_timeout() - cachedTime, rak::timer()).usec();
-}
-
-int16_t
-Download::tracker_numwant() const {
-  return m_ptr->main()->tracker_manager()->tracker_info()->get_numwant();
-}
-
 void
 Download::set_peers_min(uint32_t v) {
   if (v > (1 << 16))
@@ -302,78 +326,9 @@ Download::set_uploads_max(uint32_t v) {
   m_ptr->main()->choke_manager()->balance();
 }
 
-void
-Download::set_tracker_numwant(int32_t n) {
-  m_ptr->main()->tracker_manager()->tracker_info()->set_numwant(n);
-}
-
-Tracker
-Download::tracker(uint32_t index) {
-  if (index >= m_ptr->main()->tracker_manager()->size())
-    throw client_error("Client called Download::get_tracker(...) with out of range index");
-
-  return m_ptr->main()->tracker_manager()->get(index);
-}
-
-const Tracker
-Download::tracker(uint32_t index) const {
-  if (index >= m_ptr->main()->tracker_manager()->size())
-    throw client_error("Client called Download::get_tracker(...) with out of range index");
-
-  return m_ptr->main()->tracker_manager()->get(index);
-}
-
-uint32_t
-Download::size_trackers() const {
-  return m_ptr->main()->tracker_manager()->size();
-}
-
-uint32_t
-Download::tracker_focus() const {
-  return m_ptr->main()->tracker_manager()->focus_index();
-}
-
-void
-Download::tracker_send_completed() {
-  m_ptr->main()->tracker_manager()->send_completed();
-}
-
-void
-Download::tracker_cycle_group(int group) {
-  m_ptr->main()->tracker_manager()->cycle_group(group);
-}
-
-void
-Download::tracker_manual_request(bool force) {
-  m_ptr->main()->tracker_manager()->manual_request(force);
-}
-
-Entry
-Download::file_entry(uint32_t index) {
-  if (index >= m_ptr->main()->content()->entry_list()->files_size())
-    throw client_error("Client called Download::get_entry(...) with out of range index");
-
-  return m_ptr->main()->content()->entry_list()->get_node(index);
-}
-
-bool
-Download::file_entry_created(__UNUSED uint32_t index) {
-  return true;
-}
-
-uint32_t
-Download::size_file_entries() const {
-  return m_ptr->main()->content()->entry_list()->files_size();
-}
-
-const Download::SeenVector&
-Download::seen_chunks() const {
-  return m_ptr->main()->chunk_selector()->bitfield_counter()->field();
-}
-
 Download::ConnectionType
 Download::connection_type() const {
-  return (ConnectionType)m_ptr->get_connection_type();
+  return (ConnectionType)m_ptr->connection_type();
 }
 
 void
@@ -410,66 +365,70 @@ Download::peer_find(const std::string& id) {
     std::find_if(m_ptr->main()->connection_list()->begin(),
 		 m_ptr->main()->connection_list()->end(),
 		 
-		 rak::equal(id, rak::on(std::mem_fun(&PeerConnectionBase::get_peer),
-					std::mem_fun_ref(&PeerInfo::get_id))));
+		 rak::equal(id, rak::on(std::mem_fun(&PeerConnectionBase::peer_info),
+					std::mem_fun(&PeerInfo::get_id))));
 
   return itr != m_ptr->main()->connection_list()->end() ? *itr : NULL;
 }
 
+void
+Download::disconnect_peer(Peer p) {
+  m_ptr->main()->connection_list()->erase(p.ptr());
+}
+
 sigc::connection
-Download::signal_download_done(Download::SlotVoid s) {
+Download::signal_download_done(Download::slot_void_type s) {
   return m_ptr->signal_download_done().connect(s);
 }
 
 sigc::connection
-Download::signal_hash_done(Download::SlotVoid s) {
+Download::signal_hash_done(Download::slot_void_type s) {
   return m_ptr->signal_initial_hash().connect(s);
 }
 
 sigc::connection
-Download::signal_peer_connected(Download::SlotPeer s) {
+Download::signal_peer_connected(Download::slot_peer_type s) {
   return m_ptr->signal_peer_connected().connect(s);
 }
 
 sigc::connection
-Download::signal_peer_disconnected(Download::SlotPeer s) {
+Download::signal_peer_disconnected(Download::slot_peer_type s) {
   return m_ptr->signal_peer_disconnected().connect(s);
 }
 
 sigc::connection
-Download::signal_tracker_succeded(Download::SlotVoid s) {
+Download::signal_tracker_succeded(Download::slot_void_type s) {
   return m_ptr->signal_tracker_success().connect(s);
 }
 
 sigc::connection
-Download::signal_tracker_failed(Download::SlotString s) {
+Download::signal_tracker_failed(Download::slot_string_type s) {
   return m_ptr->signal_tracker_failed().connect(s);
 }
 
 sigc::connection
-Download::signal_tracker_dump(__UNUSED Download::SlotIStream s) {
-//   return m_ptr->main()->tracker_manager()->signal_dump().connect(s);
-  return sigc::connection();
+Download::signal_tracker_dump(Download::slot_dump_type s) {
+  return m_ptr->info()->signal_tracker_dump().connect(s);
 }
 
 sigc::connection
-Download::signal_chunk_passed(Download::SlotChunk s) {
+Download::signal_chunk_passed(Download::slot_chunk_type s) {
   return m_ptr->signal_chunk_passed().connect(s);
 }
 
 sigc::connection
-Download::signal_chunk_failed(Download::SlotChunk s) {
+Download::signal_chunk_failed(Download::slot_chunk_type s) {
   return m_ptr->signal_chunk_failed().connect(s);
 }
 
 sigc::connection
-Download::signal_network_log(SlotString s) {
-  return m_ptr->main()->signal_network_log().connect(s);
+Download::signal_network_log(slot_string_type s) {
+  return m_ptr->info()->signal_network_log().connect(s);
 }
 
 sigc::connection
-Download::signal_storage_error(SlotString s) {
-  return m_ptr->main()->signal_storage_error().connect(s);
+Download::signal_storage_error(slot_string_type s) {
+  return m_ptr->info()->signal_storage_error().connect(s);
 }
 
 }
