@@ -47,32 +47,32 @@ namespace torrent {
 
 bool
 Peer::is_incoming() const {
-  return m_ptr->get_peer().is_incoming();
+  return m_ptr->peer_info()->is_incoming();
 }
 
 bool
-Peer::is_local_choked() {
+Peer::is_local_choked() const {
   return m_ptr->is_up_choked();
 }
 
 bool
-Peer::is_local_interested() {
+Peer::is_local_interested() const {
   return m_ptr->is_up_interested();
 }
 
 bool
-Peer::is_remote_choked() {
+Peer::is_remote_choked() const {
   return m_ptr->is_down_choked();
 }
 
 bool
-Peer::is_remote_interested() {
+Peer::is_remote_interested() const {
   return m_ptr->is_down_interested();
 }
 
 bool
-Peer::is_snubbed() {
-  return m_ptr->is_snubbed();
+Peer::is_snubbed() const {
+  return m_ptr->peer_chunks()->is_snubbed();
 }
 
 void
@@ -80,79 +80,62 @@ Peer::set_snubbed(bool v) {
   m_ptr->set_snubbed(v);
 }
 
-std::string
-Peer::id() {
-  return m_ptr->get_peer().get_id();
-}
-
-std::string
-Peer::address() {
-  return m_ptr->get_peer().get_address();
-}
-
-uint16_t
-Peer::port() {
-  return m_ptr->get_peer().get_port();
+const std::string&
+Peer::id() const {
+  return m_ptr->peer_info()->get_id();
 }
 
 const char*
-Peer::options() {
-  return m_ptr->get_peer().get_options();
+Peer::options() const {
+  return m_ptr->peer_info()->get_options();
+}
+
+const sockaddr*
+Peer::address() const {
+  return m_ptr->peer_info()->socket_address()->c_sockaddr();
 }
 
 const Rate*
-Peer::down_rate() {
-  return m_ptr->down_rate();
+Peer::down_rate() const {
+  return m_ptr->peer_chunks()->download_throttle()->rate();
 } 
 
 const Rate*
-Peer::up_rate() {
-  return m_ptr->up_rate();
+Peer::up_rate() const {
+  return m_ptr->peer_chunks()->upload_throttle()->rate();
 } 
 
 const Rate*
-Peer::peer_rate() {
-  return m_ptr->peer_rate();
+Peer::peer_rate() const {
+  return m_ptr->peer_chunks()->peer_rate();
 } 
 
-uint32_t
-Peer::incoming_queue_size() {
-  return m_ptr->get_request_list().size();
+const Bitfield*
+Peer::bitfield() const {
+  return m_ptr->peer_chunks()->bitfield();
+}
+
+const Piece*
+Peer::incoming_queue(uint32_t pos) const {
+  if (pos >= m_ptr->download_queue()->size())
+    throw client_error("Peer::incoming_queue(...) out of range.");
+
+  return m_ptr->download_queue()->queued_piece(pos);
 }
 
 uint32_t
-Peer::outgoing_queue_size() {
-  return m_ptr->get_send_list().size();
+Peer::incoming_queue_size() const {
+  return m_ptr->download_queue()->size();
+}
+
+uint32_t
+Peer::outgoing_queue_size() const {
+  return m_ptr->peer_chunks()->upload_queue()->size();
 }  
 
 uint32_t
-Peer::incoming_index(uint32_t pos) {
-  if (pos >= m_ptr->get_request_list().size())
-    throw client_error("get_incoming_index(pos) out of range");
-
-  return m_ptr->get_request_list().get_queued_piece(pos).get_index();
-}
-
-  uint32_t             incoming_offset(uint32_t pos);
-  uint32_t             incoming_length(uint32_t pos);
-
-// Currently needs to copy the data once to a std::string. But 
-// since gcc does ref counted std::string, you can inexpensively
-// copy the resulting string. Will consider making BitField use a
-// std::string.
-const unsigned char*
-Peer::bitfield_data() {
-  return (const unsigned char*)m_ptr->peer_chunks()->bitfield()->begin();
-}
-
-uint32_t
-Peer::bitfield_size() {
-  return m_ptr->peer_chunks()->bitfield()->size_bits();
-}
-
-uint32_t
-Peer::chunks_done() {
-  return m_ptr->peer_chunks()->bitfield()->count();
+Peer::chunks_done() const {
+  return m_ptr->peer_chunks()->bitfield()->size_set();
 }  
 
 }
