@@ -38,16 +38,22 @@
 
 #include <iomanip>
 #include <sstream>
+#include <rak/functional.h>
 #include <rak/string_manip.h>
 
+#include "torrent/connection_manager.h"
 #include "torrent/exceptions.h"
 #include "torrent/http.h"
+#include "torrent/object_stream.h"
+
 #include "tracker_http.h"
-#include "tracker_info.h"
+
+#include "globals.h"
+#include "manager.h"
 
 namespace torrent {
 
-TrackerHttp::TrackerHttp(TrackerInfo* info, const std::string& url) :
+TrackerHttp::TrackerHttp(DownloadInfo* info, const std::string& url) :
   TrackerBase(info, url),
 
   m_get(Http::call_factory()),
@@ -68,49 +74,56 @@ TrackerHttp::is_busy() const {
 }
 
 void
-TrackerHttp::send_state(TrackerInfo::State state, uint64_t down, uint64_t up, uint64_t left) {
+TrackerHttp::send_state(DownloadInfo::State state, uint64_t down, uint64_t up, uint64_t left) {
   close();
 
   if (m_info == NULL)
     throw internal_error("TrackerHttp::send_state(...) does not have a valid m_info");
 
-  if (m_info->get_local_id().length() != 20 ||
-      m_info->local_address().get_port() == 0 ||
-      m_info->get_hash().length() != 20)
-    throw internal_error("Send state with TrackerHttp with bad hash, id or port");
+  if (m_info->local_id().length() != 20 ||
+      m_info->hash().length() != 20)
+    throw internal_error("Send state with TrackerHttp with bad hash or id");
 
   std::stringstream s;
+  s.imbue(std::locale::classic());
 
   s << m_url
-    << "?info_hash=" << rak::copy_escape_html(m_info->get_hash())
-    << "&peer_id=" << rak::copy_escape_html(m_info->get_local_id())
-    << "&key=" << std::hex << std::setw(8) << std::setfill('0') << m_info->get_key() << std::dec;
+    << "?info_hash=" << rak::copy_escape_html(m_info->hash())
+    << "&peer_id=" << rak::copy_escape_html(m_info->local_id());
+
+  if (m_info->key())
+    s << "&key=" << std::hex << std::setw(8) << std::setfill('0') << m_info->key() << std::dec;
 
   if (!m_trackerId.empty())
     s << "&trackerid=" << rak::copy_escape_html(m_trackerId);
 
-  if (!m_info->local_address().is_address_any())
-    s << "&ip=" << m_info->local_address().get_address();
+  const rak::socket_address* localAddress = rak::socket_address::cast_from(manager->connection_manager()->local_address());
 
-  if (m_info->get_compact())
+  if (localAddress->family() == rak::socket_address::af_inet &&
+      !localAddress->sa_inet()->is_address_any())
+    s << "&ip=" << localAddress->address_str();
+
+  if (m_info->compact())
     s << "&compact=1";
 
-  if (m_info->get_numwant() >= 0)
-    s << "&numwant=" << m_info->get_numwant();
+  if (m_info->numwant() >= 0)
+    s << "&numwant=" << m_info->numwant();
 
-  s << "&port=" << m_info->local_address().get_port()
-    << "&uploaded=" << up
+  if (manager->connection_manager()->listen_port())
+    s << "&port=" << manager->connection_manager()->listen_port();
+
+  s << "&uploaded=" << up
     << "&downloaded=" << down
     << "&left=" << left;
 
   switch(state) {
-  case TrackerInfo::STARTED:
+  case DownloadInfo::STARTED:
     s << "&event=started";
     break;
-  case TrackerInfo::STOPPED:
+  case DownloadInfo::STOPPED:
     s << "&event=stopped";
     break;
-  case TrackerInfo::COMPLETED:
+  case DownloadInfo::COMPLETED:
     s << "&event=completed";
     break;
   default:
@@ -139,7 +152,7 @@ TrackerHttp::close() {
 }
 
 TrackerHttp::Type
-TrackerHttp::get_type() const {
+TrackerHttp::type() const {
   return TRACKER_HTTP;
 }
 
@@ -148,7 +161,13 @@ TrackerHttp::receive_done() {
   if (m_data == NULL)
     throw internal_error("TrackerHttp::receive_done() called on an invalid object");
 
-  Bencode b;
+  if (!m_info->signal_tracker_dump().empty()) {
+    std::string dump = m_data->str();
+
+    m_info->signal_tracker_dump().emit(m_get->url(), dump.c_str(), dump.size());
+  }
+
+  Object b;
   *m_data >> b;
 
   if (m_data->fail())
@@ -164,22 +183,35 @@ TrackerHttp::receive_done() {
 			  std::string("failure reason not a string"))
 			 + "\"");
 
-  if (b.has_key("interval") && b.get_key("interval").is_value())
+  if (b.has_key_value("interval"))
     m_slotSetInterval(b.get_key("interval").as_value());
   
-  if (b.has_key("min interval") && b.get_key("min interval").is_value())
+  if (b.has_key_value("min interval"))
     m_slotSetMinInterval(b.get_key("min interval").as_value());
 
-  if (b.has_key("tracker id") && b.get_key("tracker id").is_string())
+  if (b.has_key_string("tracker id"))
     m_trackerId = b.get_key("tracker id").as_string();
 
+  if (b.has_key_value("complete") && b.has_key_value("incomplete")) {
+    m_scrapeComplete   = std::max<int64_t>(b.get_key("complete").as_value(), 0);
+    m_scrapeIncomplete = std::max<int64_t>(b.get_key("incomplete").as_value(), 0);
+
+    m_scrapeTimeLast = cachedTime;
+  }
+
+  if (b.has_key_value("downloaded"))
+    m_scrapeDownloaded = std::max<int64_t>(b.get_key("downloaded").as_value(), 0);
+
   AddressList l;
 
   try {
+    // Due to some trackers sending the wrong type when no peers are
+    // available, don't bork on it.
     if (b.get_key("peers").is_string())
-      parse_address_compact(l, b.get_key("peers").as_string());
-    else
-      parse_address_normal(l, b.get_key("peers").as_list());
+      parse_address_compact(&l, b.get_key("peers").as_string());
+
+    else if (b.get_key("peers").is_list())
+      parse_address_normal(&l, b.get_key("peers").as_list());
 
   } catch (bencode_error& e) {
     return receive_failed(e.what());
@@ -196,37 +228,38 @@ TrackerHttp::receive_failed(std::string msg) {
   m_slotFailed(this, msg);
 }
 
-inline SocketAddress
-TrackerHttp::parse_address(const Bencode& b) {
-  SocketAddress sa;
+inline rak::socket_address
+TrackerHttp::parse_address(const Object& b) {
+  rak::socket_address sa;
+  sa.clear();
 
   if (!b.is_map())
-    return SocketAddress();
+    return sa;
 
-  for (Bencode::Map::const_iterator itr = b.as_map().begin(); itr != b.as_map().end(); ++itr)
-    // We ignore the "peer id" field, we don't need it.
-    if (itr->first == "ip" && itr->second.is_string())
-      sa.set_address(itr->second.as_string());
+  if (!b.has_key_string("ip") || !sa.set_address_str(b.get_key("ip").as_string()))
+    return sa;
 
-    else if (itr->first == "port" && itr->second.is_value())
-      sa.set_port(itr->second.as_value());
+  if (!b.has_key_value("port") || b.get_key("port").as_value() <= 0 || b.get_key("port").as_value() >= (1 << 16))
+    return sa;
 
-  return sa.is_valid() ? sa : SocketAddress();
+  sa.set_port(b.get_key("port").as_value());
+
+  return sa;
 }
 
 void
-TrackerHttp::parse_address_normal(AddressList& l, const Bencode::List& b) {
-  std::transform(b.begin(), b.end(), std::back_inserter(l), std::ptr_fun(&TrackerHttp::parse_address));
-}  
+TrackerHttp::parse_address_normal(AddressList* l, const Object::list_type& b) {
+  std::for_each(b.begin(), b.end(), rak::on(std::ptr_fun(&TrackerHttp::parse_address), address_list_add_address(l)));
+}
 
 void
-TrackerHttp::parse_address_compact(AddressList& l, const std::string& s) {
+TrackerHttp::parse_address_compact(AddressList* l, const std::string& s) {
   if (sizeof(const SocketAddressCompact) != 6)
     throw internal_error("TrackerHttp::parse_address_compact(...) bad struct size.");
 
   std::copy(reinterpret_cast<const SocketAddressCompact*>(s.c_str()),
 	    reinterpret_cast<const SocketAddressCompact*>(s.c_str() + s.size() - s.size() % sizeof(SocketAddressCompact)),
-	    std::back_inserter(l));
+	    std::back_inserter(*l));
 }
 
 }
