@@ -40,21 +40,21 @@
 #include <list>
 #include <rak/functional.h>
 
-#include "net/socket_address.h"
+#include <rak/socket_address.h>
 #include "globals.h"
 
 namespace torrent {
 
 class DownloadWrapper;
 class TrackerControl;
-class TrackerInfo;
+class DownloadInfo;
 class TrackerBase;
 
 class TrackerManager {
 public:
   typedef uint32_t                                size_type;
   typedef std::pair<int, TrackerBase*>            value_type;
-  typedef std::list<SocketAddress>                AddressList;
+  typedef std::list<rak::socket_address>          AddressList;
 
   typedef rak::mem_fun1<DownloadWrapper, void, AddressList*>       SlotSuccess;
   typedef rak::mem_fun1<DownloadWrapper, void, const std::string&> SlotFailed;
@@ -91,7 +91,9 @@ public:
 
   void                insert(int group, const std::string& url);
 
-  TrackerInfo*        tracker_info();
+  DownloadInfo*       info();
+  const DownloadInfo* info() const;
+  void                set_info(DownloadInfo* info);
 
   rak::timer          get_next_timeout() const                  { return m_taskTimeout.time(); }
 
